from http import HTTPStatus
from typing import Any, Dict, List, Optional, Union

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response, UNSET
from ... import errors

from ...models.session_dto import SessionDto
from ...types import Unset
import datetime
from ...models.longship_error import LongshipError
from ...models.get_all_sessions_order_by import GetAllSessionsOrderBy


def _get_kwargs(
    *,
    skip: Union[Unset, int] = UNSET,
    take: Union[Unset, int] = UNSET,
    search: Union[Unset, str] = UNSET,
    connector_number: Union[Unset, int] = UNSET,
    chargepoint_id: Union[Unset, str] = UNSET,
    running_only: Union[Unset, bool] = UNSET,
    completed_only: Union[Unset, bool] = UNSET,
    suspicious_only: Union[Unset, bool] = UNSET,
    rejected_only: Union[Unset, bool] = UNSET,
    local_only: Union[Unset, bool] = UNSET,
    unauthorized_only: Union[Unset, bool] = UNSET,
    roamingconnectionid: Union[Unset, str] = UNSET,
    organizationunitcode: Union[Unset, str] = UNSET,
    tariff_id: Union[Unset, str] = UNSET,
    pricegreaterthen: Union[Unset, float] = UNSET,
    pricelessthen: Union[Unset, float] = UNSET,
    kwhgreaterthen: Union[Unset, float] = UNSET,
    kwhlessthen: Union[Unset, float] = UNSET,
    contractid: Union[Unset, str] = UNSET,
    providerid: Union[Unset, str] = UNSET,
    authorizationreferenceid: Union[Unset, str] = UNSET,
    from_: Union[Unset, datetime.datetime] = UNSET,
    to: Union[Unset, datetime.datetime] = UNSET,
    order_by: Union[Unset, GetAllSessionsOrderBy] = GetAllSessionsOrderBy.START,
    descending: Union[Unset, bool] = UNSET,
) -> Dict[str, Any]:
    params: Dict[str, Any] = {}

    params["skip"] = skip

    params["take"] = take

    params["search"] = search

    params["connectorNumber"] = connector_number

    params["chargepointId"] = chargepoint_id

    params["runningOnly"] = running_only

    params["completedOnly"] = completed_only

    params["suspiciousOnly"] = suspicious_only

    params["rejectedOnly"] = rejected_only

    params["localOnly"] = local_only

    params["unauthorizedOnly"] = unauthorized_only

    params["roamingconnectionid"] = roamingconnectionid

    params["organizationunitcode"] = organizationunitcode

    params["tariffId"] = tariff_id

    params["pricegreaterthen"] = pricegreaterthen

    params["pricelessthen"] = pricelessthen

    params["kwhgreaterthen"] = kwhgreaterthen

    params["kwhlessthen"] = kwhlessthen

    params["contractid"] = contractid

    params["providerid"] = providerid

    params["authorizationreferenceid"] = authorizationreferenceid

    json_from_: Union[Unset, str] = UNSET
    if not isinstance(from_, Unset):
        json_from_ = from_.isoformat()
    params["from"] = json_from_

    json_to: Union[Unset, str] = UNSET
    if not isinstance(to, Unset):
        json_to = to.isoformat()
    params["to"] = json_to

    json_order_by: Union[Unset, str] = UNSET
    if not isinstance(order_by, Unset):
        json_order_by = order_by.value

    params["orderBy"] = json_order_by

    params["descending"] = descending

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: Dict[str, Any] = {
        "method": "get",
        "url": "/v1/sessions",
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[List["SessionDto"], LongshipError]]:
    if response.status_code == HTTPStatus.OK:
        response_200 = []
        _response_200 = response.json()
        for componentsschemassession_dto_array_item_data in _response_200:
            componentsschemassession_dto_array_item = SessionDto.from_dict(
                componentsschemassession_dto_array_item_data
            )

            response_200.append(componentsschemassession_dto_array_item)

        return response_200
    if response.status_code == HTTPStatus.UNAUTHORIZED:
        response_401 = LongshipError.from_dict(response.json())

        return response_401
    if response.status_code == HTTPStatus.INTERNAL_SERVER_ERROR:
        response_500 = LongshipError.from_dict(response.json())

        return response_500
    if response.status_code == HTTPStatus.FORBIDDEN:
        response_403 = LongshipError.from_dict(response.json())

        return response_403
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[List["SessionDto"], LongshipError]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    skip: Union[Unset, int] = UNSET,
    take: Union[Unset, int] = UNSET,
    search: Union[Unset, str] = UNSET,
    connector_number: Union[Unset, int] = UNSET,
    chargepoint_id: Union[Unset, str] = UNSET,
    running_only: Union[Unset, bool] = UNSET,
    completed_only: Union[Unset, bool] = UNSET,
    suspicious_only: Union[Unset, bool] = UNSET,
    rejected_only: Union[Unset, bool] = UNSET,
    local_only: Union[Unset, bool] = UNSET,
    unauthorized_only: Union[Unset, bool] = UNSET,
    roamingconnectionid: Union[Unset, str] = UNSET,
    organizationunitcode: Union[Unset, str] = UNSET,
    tariff_id: Union[Unset, str] = UNSET,
    pricegreaterthen: Union[Unset, float] = UNSET,
    pricelessthen: Union[Unset, float] = UNSET,
    kwhgreaterthen: Union[Unset, float] = UNSET,
    kwhlessthen: Union[Unset, float] = UNSET,
    contractid: Union[Unset, str] = UNSET,
    providerid: Union[Unset, str] = UNSET,
    authorizationreferenceid: Union[Unset, str] = UNSET,
    from_: Union[Unset, datetime.datetime] = UNSET,
    to: Union[Unset, datetime.datetime] = UNSET,
    order_by: Union[Unset, GetAllSessionsOrderBy] = GetAllSessionsOrderBy.START,
    descending: Union[Unset, bool] = UNSET,
) -> Response[Union[List["SessionDto"], LongshipError]]:
    """Get a list of sessions.

     Get a paged list of sessions, taken the filters into account.

    Args:
        skip (Union[Unset, int]):
        take (Union[Unset, int]):
        search (Union[Unset, str]):
        connector_number (Union[Unset, int]):
        chargepoint_id (Union[Unset, str]):
        running_only (Union[Unset, bool]):
        completed_only (Union[Unset, bool]):
        suspicious_only (Union[Unset, bool]):
        rejected_only (Union[Unset, bool]):
        local_only (Union[Unset, bool]):
        unauthorized_only (Union[Unset, bool]):
        roamingconnectionid (Union[Unset, str]):
        organizationunitcode (Union[Unset, str]):
        tariff_id (Union[Unset, str]):
        pricegreaterthen (Union[Unset, float]):
        pricelessthen (Union[Unset, float]):
        kwhgreaterthen (Union[Unset, float]):
        kwhlessthen (Union[Unset, float]):
        contractid (Union[Unset, str]):
        providerid (Union[Unset, str]):
        authorizationreferenceid (Union[Unset, str]):
        from_ (Union[Unset, datetime.datetime]):
        to (Union[Unset, datetime.datetime]):
        order_by (Union[Unset, GetAllSessionsOrderBy]):  Default: GetAllSessionsOrderBy.START.
        descending (Union[Unset, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[List['SessionDto'], LongshipError]]
    """

    kwargs = _get_kwargs(
        skip=skip,
        take=take,
        search=search,
        connector_number=connector_number,
        chargepoint_id=chargepoint_id,
        running_only=running_only,
        completed_only=completed_only,
        suspicious_only=suspicious_only,
        rejected_only=rejected_only,
        local_only=local_only,
        unauthorized_only=unauthorized_only,
        roamingconnectionid=roamingconnectionid,
        organizationunitcode=organizationunitcode,
        tariff_id=tariff_id,
        pricegreaterthen=pricegreaterthen,
        pricelessthen=pricelessthen,
        kwhgreaterthen=kwhgreaterthen,
        kwhlessthen=kwhlessthen,
        contractid=contractid,
        providerid=providerid,
        authorizationreferenceid=authorizationreferenceid,
        from_=from_,
        to=to,
        order_by=order_by,
        descending=descending,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: Union[AuthenticatedClient, Client],
    skip: Union[Unset, int] = UNSET,
    take: Union[Unset, int] = UNSET,
    search: Union[Unset, str] = UNSET,
    connector_number: Union[Unset, int] = UNSET,
    chargepoint_id: Union[Unset, str] = UNSET,
    running_only: Union[Unset, bool] = UNSET,
    completed_only: Union[Unset, bool] = UNSET,
    suspicious_only: Union[Unset, bool] = UNSET,
    rejected_only: Union[Unset, bool] = UNSET,
    local_only: Union[Unset, bool] = UNSET,
    unauthorized_only: Union[Unset, bool] = UNSET,
    roamingconnectionid: Union[Unset, str] = UNSET,
    organizationunitcode: Union[Unset, str] = UNSET,
    tariff_id: Union[Unset, str] = UNSET,
    pricegreaterthen: Union[Unset, float] = UNSET,
    pricelessthen: Union[Unset, float] = UNSET,
    kwhgreaterthen: Union[Unset, float] = UNSET,
    kwhlessthen: Union[Unset, float] = UNSET,
    contractid: Union[Unset, str] = UNSET,
    providerid: Union[Unset, str] = UNSET,
    authorizationreferenceid: Union[Unset, str] = UNSET,
    from_: Union[Unset, datetime.datetime] = UNSET,
    to: Union[Unset, datetime.datetime] = UNSET,
    order_by: Union[Unset, GetAllSessionsOrderBy] = GetAllSessionsOrderBy.START,
    descending: Union[Unset, bool] = UNSET,
) -> Optional[Union[List["SessionDto"], LongshipError]]:
    """Get a list of sessions.

     Get a paged list of sessions, taken the filters into account.

    Args:
        skip (Union[Unset, int]):
        take (Union[Unset, int]):
        search (Union[Unset, str]):
        connector_number (Union[Unset, int]):
        chargepoint_id (Union[Unset, str]):
        running_only (Union[Unset, bool]):
        completed_only (Union[Unset, bool]):
        suspicious_only (Union[Unset, bool]):
        rejected_only (Union[Unset, bool]):
        local_only (Union[Unset, bool]):
        unauthorized_only (Union[Unset, bool]):
        roamingconnectionid (Union[Unset, str]):
        organizationunitcode (Union[Unset, str]):
        tariff_id (Union[Unset, str]):
        pricegreaterthen (Union[Unset, float]):
        pricelessthen (Union[Unset, float]):
        kwhgreaterthen (Union[Unset, float]):
        kwhlessthen (Union[Unset, float]):
        contractid (Union[Unset, str]):
        providerid (Union[Unset, str]):
        authorizationreferenceid (Union[Unset, str]):
        from_ (Union[Unset, datetime.datetime]):
        to (Union[Unset, datetime.datetime]):
        order_by (Union[Unset, GetAllSessionsOrderBy]):  Default: GetAllSessionsOrderBy.START.
        descending (Union[Unset, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[List['SessionDto'], LongshipError]
    """

    return sync_detailed(
        client=client,
        skip=skip,
        take=take,
        search=search,
        connector_number=connector_number,
        chargepoint_id=chargepoint_id,
        running_only=running_only,
        completed_only=completed_only,
        suspicious_only=suspicious_only,
        rejected_only=rejected_only,
        local_only=local_only,
        unauthorized_only=unauthorized_only,
        roamingconnectionid=roamingconnectionid,
        organizationunitcode=organizationunitcode,
        tariff_id=tariff_id,
        pricegreaterthen=pricegreaterthen,
        pricelessthen=pricelessthen,
        kwhgreaterthen=kwhgreaterthen,
        kwhlessthen=kwhlessthen,
        contractid=contractid,
        providerid=providerid,
        authorizationreferenceid=authorizationreferenceid,
        from_=from_,
        to=to,
        order_by=order_by,
        descending=descending,
    ).parsed


async def asyncio_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    skip: Union[Unset, int] = UNSET,
    take: Union[Unset, int] = UNSET,
    search: Union[Unset, str] = UNSET,
    connector_number: Union[Unset, int] = UNSET,
    chargepoint_id: Union[Unset, str] = UNSET,
    running_only: Union[Unset, bool] = UNSET,
    completed_only: Union[Unset, bool] = UNSET,
    suspicious_only: Union[Unset, bool] = UNSET,
    rejected_only: Union[Unset, bool] = UNSET,
    local_only: Union[Unset, bool] = UNSET,
    unauthorized_only: Union[Unset, bool] = UNSET,
    roamingconnectionid: Union[Unset, str] = UNSET,
    organizationunitcode: Union[Unset, str] = UNSET,
    tariff_id: Union[Unset, str] = UNSET,
    pricegreaterthen: Union[Unset, float] = UNSET,
    pricelessthen: Union[Unset, float] = UNSET,
    kwhgreaterthen: Union[Unset, float] = UNSET,
    kwhlessthen: Union[Unset, float] = UNSET,
    contractid: Union[Unset, str] = UNSET,
    providerid: Union[Unset, str] = UNSET,
    authorizationreferenceid: Union[Unset, str] = UNSET,
    from_: Union[Unset, datetime.datetime] = UNSET,
    to: Union[Unset, datetime.datetime] = UNSET,
    order_by: Union[Unset, GetAllSessionsOrderBy] = GetAllSessionsOrderBy.START,
    descending: Union[Unset, bool] = UNSET,
) -> Response[Union[List["SessionDto"], LongshipError]]:
    """Get a list of sessions.

     Get a paged list of sessions, taken the filters into account.

    Args:
        skip (Union[Unset, int]):
        take (Union[Unset, int]):
        search (Union[Unset, str]):
        connector_number (Union[Unset, int]):
        chargepoint_id (Union[Unset, str]):
        running_only (Union[Unset, bool]):
        completed_only (Union[Unset, bool]):
        suspicious_only (Union[Unset, bool]):
        rejected_only (Union[Unset, bool]):
        local_only (Union[Unset, bool]):
        unauthorized_only (Union[Unset, bool]):
        roamingconnectionid (Union[Unset, str]):
        organizationunitcode (Union[Unset, str]):
        tariff_id (Union[Unset, str]):
        pricegreaterthen (Union[Unset, float]):
        pricelessthen (Union[Unset, float]):
        kwhgreaterthen (Union[Unset, float]):
        kwhlessthen (Union[Unset, float]):
        contractid (Union[Unset, str]):
        providerid (Union[Unset, str]):
        authorizationreferenceid (Union[Unset, str]):
        from_ (Union[Unset, datetime.datetime]):
        to (Union[Unset, datetime.datetime]):
        order_by (Union[Unset, GetAllSessionsOrderBy]):  Default: GetAllSessionsOrderBy.START.
        descending (Union[Unset, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[List['SessionDto'], LongshipError]]
    """

    kwargs = _get_kwargs(
        skip=skip,
        take=take,
        search=search,
        connector_number=connector_number,
        chargepoint_id=chargepoint_id,
        running_only=running_only,
        completed_only=completed_only,
        suspicious_only=suspicious_only,
        rejected_only=rejected_only,
        local_only=local_only,
        unauthorized_only=unauthorized_only,
        roamingconnectionid=roamingconnectionid,
        organizationunitcode=organizationunitcode,
        tariff_id=tariff_id,
        pricegreaterthen=pricegreaterthen,
        pricelessthen=pricelessthen,
        kwhgreaterthen=kwhgreaterthen,
        kwhlessthen=kwhlessthen,
        contractid=contractid,
        providerid=providerid,
        authorizationreferenceid=authorizationreferenceid,
        from_=from_,
        to=to,
        order_by=order_by,
        descending=descending,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: Union[AuthenticatedClient, Client],
    skip: Union[Unset, int] = UNSET,
    take: Union[Unset, int] = UNSET,
    search: Union[Unset, str] = UNSET,
    connector_number: Union[Unset, int] = UNSET,
    chargepoint_id: Union[Unset, str] = UNSET,
    running_only: Union[Unset, bool] = UNSET,
    completed_only: Union[Unset, bool] = UNSET,
    suspicious_only: Union[Unset, bool] = UNSET,
    rejected_only: Union[Unset, bool] = UNSET,
    local_only: Union[Unset, bool] = UNSET,
    unauthorized_only: Union[Unset, bool] = UNSET,
    roamingconnectionid: Union[Unset, str] = UNSET,
    organizationunitcode: Union[Unset, str] = UNSET,
    tariff_id: Union[Unset, str] = UNSET,
    pricegreaterthen: Union[Unset, float] = UNSET,
    pricelessthen: Union[Unset, float] = UNSET,
    kwhgreaterthen: Union[Unset, float] = UNSET,
    kwhlessthen: Union[Unset, float] = UNSET,
    contractid: Union[Unset, str] = UNSET,
    providerid: Union[Unset, str] = UNSET,
    authorizationreferenceid: Union[Unset, str] = UNSET,
    from_: Union[Unset, datetime.datetime] = UNSET,
    to: Union[Unset, datetime.datetime] = UNSET,
    order_by: Union[Unset, GetAllSessionsOrderBy] = GetAllSessionsOrderBy.START,
    descending: Union[Unset, bool] = UNSET,
) -> Optional[Union[List["SessionDto"], LongshipError]]:
    """Get a list of sessions.

     Get a paged list of sessions, taken the filters into account.

    Args:
        skip (Union[Unset, int]):
        take (Union[Unset, int]):
        search (Union[Unset, str]):
        connector_number (Union[Unset, int]):
        chargepoint_id (Union[Unset, str]):
        running_only (Union[Unset, bool]):
        completed_only (Union[Unset, bool]):
        suspicious_only (Union[Unset, bool]):
        rejected_only (Union[Unset, bool]):
        local_only (Union[Unset, bool]):
        unauthorized_only (Union[Unset, bool]):
        roamingconnectionid (Union[Unset, str]):
        organizationunitcode (Union[Unset, str]):
        tariff_id (Union[Unset, str]):
        pricegreaterthen (Union[Unset, float]):
        pricelessthen (Union[Unset, float]):
        kwhgreaterthen (Union[Unset, float]):
        kwhlessthen (Union[Unset, float]):
        contractid (Union[Unset, str]):
        providerid (Union[Unset, str]):
        authorizationreferenceid (Union[Unset, str]):
        from_ (Union[Unset, datetime.datetime]):
        to (Union[Unset, datetime.datetime]):
        order_by (Union[Unset, GetAllSessionsOrderBy]):  Default: GetAllSessionsOrderBy.START.
        descending (Union[Unset, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[List['SessionDto'], LongshipError]
    """

    return (
        await asyncio_detailed(
            client=client,
            skip=skip,
            take=take,
            search=search,
            connector_number=connector_number,
            chargepoint_id=chargepoint_id,
            running_only=running_only,
            completed_only=completed_only,
            suspicious_only=suspicious_only,
            rejected_only=rejected_only,
            local_only=local_only,
            unauthorized_only=unauthorized_only,
            roamingconnectionid=roamingconnectionid,
            organizationunitcode=organizationunitcode,
            tariff_id=tariff_id,
            pricegreaterthen=pricegreaterthen,
            pricelessthen=pricelessthen,
            kwhgreaterthen=kwhgreaterthen,
            kwhlessthen=kwhlessthen,
            contractid=contractid,
            providerid=providerid,
            authorizationreferenceid=authorizationreferenceid,
            from_=from_,
            to=to,
            order_by=order_by,
            descending=descending,
        )
    ).parsed



from enum import Enum


class StartedByTokenDtoAuthMethod(str, Enum):
    AUTH_REQUEST = "AUTH_REQUEST"
    COMMAND = "COMMAND"
    WHITELIST = "WHITELIST"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class GetAllTariffdistributionsOrderBy(str, Enum):
    ENERGY_COMPENSATION = "energY_COMPENSATION"
    NAME = "name"
    TYPE = "type"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from dateutil.parser import isoparse
import datetime
from typing import Union


T = TypeVar("T", bound="TariffDistributionHistoryDto")


@_attrs_define
class TariffDistributionHistoryDto:
    """
    Attributes:
        valid_from (Union[Unset, datetime.datetime]):
        energy_compensation (Union[Unset, float]):
        fixed_tenant_k_wh_fee (Union[Unset, float]):
        percentage_fee_customer (Union[Unset, float]):
        percentage_fee_tenant (Union[Unset, float]):
    """

    valid_from: Union[Unset, datetime.datetime] = UNSET
    energy_compensation: Union[Unset, float] = UNSET
    fixed_tenant_k_wh_fee: Union[Unset, float] = UNSET
    percentage_fee_customer: Union[Unset, float] = UNSET
    percentage_fee_tenant: Union[Unset, float] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        valid_from: Union[Unset, str] = UNSET
        if not isinstance(self.valid_from, Unset):
            valid_from = self.valid_from.isoformat()

        energy_compensation = self.energy_compensation

        fixed_tenant_k_wh_fee = self.fixed_tenant_k_wh_fee

        percentage_fee_customer = self.percentage_fee_customer

        percentage_fee_tenant = self.percentage_fee_tenant

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if valid_from is not UNSET:
            field_dict["validFrom"] = valid_from
        if energy_compensation is not UNSET:
            field_dict["energyCompensation"] = energy_compensation
        if fixed_tenant_k_wh_fee is not UNSET:
            field_dict["fixedTenantKWhFee"] = fixed_tenant_k_wh_fee
        if percentage_fee_customer is not UNSET:
            field_dict["percentageFeeCustomer"] = percentage_fee_customer
        if percentage_fee_tenant is not UNSET:
            field_dict["percentageFeeTenant"] = percentage_fee_tenant

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _valid_from = d.pop("validFrom", UNSET)
        valid_from: Union[Unset, datetime.datetime]
        if isinstance(_valid_from, Unset) or _valid_from is None:
            valid_from = UNSET
        else:
            valid_from = isoparse(_valid_from)

        energy_compensation = d.pop("energyCompensation", UNSET)

        fixed_tenant_k_wh_fee = d.pop("fixedTenantKWhFee", UNSET)

        percentage_fee_customer = d.pop("percentageFeeCustomer", UNSET)

        percentage_fee_tenant = d.pop("percentageFeeTenant", UNSET)

        tariff_distribution_history_dto = cls(
            valid_from=valid_from,
            energy_compensation=energy_compensation,
            fixed_tenant_k_wh_fee=fixed_tenant_k_wh_fee,
            percentage_fee_customer=percentage_fee_customer,
            percentage_fee_tenant=percentage_fee_tenant,
        )

        tariff_distribution_history_dto.additional_properties = d
        return tariff_distribution_history_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from dateutil.parser import isoparse
import datetime
from typing import Union
from ..models.charging_schedule_charging_rate_unit import (
    ChargingScheduleChargingRateUnit,
)

if TYPE_CHECKING:
    from ..models.charging_schedule_period import ChargingSchedulePeriod


T = TypeVar("T", bound="ChargingSchedule")


@_attrs_define
class ChargingSchedule:
    """
    Attributes:
        charging_rate_unit (ChargingScheduleChargingRateUnit):  Default: ChargingScheduleChargingRateUnit.A.
        charging_schedule_period (List['ChargingSchedulePeriod']):
        duration (Union[Unset, int]):
        start_schedule (Union[Unset, datetime.datetime]):
        min_charging_rate (Union[Unset, float]):
    """

    charging_schedule_period: List["ChargingSchedulePeriod"]
    charging_rate_unit: ChargingScheduleChargingRateUnit = (
        ChargingScheduleChargingRateUnit.A
    )
    duration: Union[Unset, int] = UNSET
    start_schedule: Union[Unset, datetime.datetime] = UNSET
    min_charging_rate: Union[Unset, float] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        charging_rate_unit = self.charging_rate_unit.value

        charging_schedule_period = []
        for charging_schedule_period_item_data in self.charging_schedule_period:
            charging_schedule_period_item = charging_schedule_period_item_data.to_dict()
            charging_schedule_period.append(charging_schedule_period_item)

        duration = self.duration

        start_schedule: Union[Unset, str] = UNSET
        if not isinstance(self.start_schedule, Unset):
            start_schedule = self.start_schedule.isoformat()

        min_charging_rate = self.min_charging_rate

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "chargingRateUnit": charging_rate_unit,
                "chargingSchedulePeriod": charging_schedule_period,
            }
        )
        if duration is not UNSET:
            field_dict["duration"] = duration
        if start_schedule is not UNSET:
            field_dict["startSchedule"] = start_schedule
        if min_charging_rate is not UNSET:
            field_dict["minChargingRate"] = min_charging_rate

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.charging_schedule_period import ChargingSchedulePeriod

        d = src_dict.copy()
        charging_rate_unit = ChargingScheduleChargingRateUnit(d.pop("chargingRateUnit"))

        charging_schedule_period = []
        _charging_schedule_period = d.pop("chargingSchedulePeriod")
        for charging_schedule_period_item_data in _charging_schedule_period:
            charging_schedule_period_item = ChargingSchedulePeriod.from_dict(
                charging_schedule_period_item_data
            )

            charging_schedule_period.append(charging_schedule_period_item)

        duration = d.pop("duration", UNSET)

        _start_schedule = d.pop("startSchedule", UNSET)
        start_schedule: Union[Unset, datetime.datetime]
        if isinstance(_start_schedule, Unset) or _start_schedule is None:
            start_schedule = UNSET
        else:
            start_schedule = isoparse(_start_schedule)

        min_charging_rate = d.pop("minChargingRate", UNSET)

        charging_schedule = cls(
            charging_rate_unit=charging_rate_unit,
            charging_schedule_period=charging_schedule_period,
            duration=duration,
            start_schedule=start_schedule,
            min_charging_rate=min_charging_rate,
        )

        charging_schedule.additional_properties = d
        return charging_schedule

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class WebhookPutDtoEventTypesItem(str, Enum):
    CDR_CREATED = "CDR_CREATED"
    CHARGEPOINT_BOOTED = "CHARGEPOINT_BOOTED"
    CONNECTIVITY_STATUS = "CONNECTIVITY_STATUS"
    LOCATION_CREATED = "LOCATION_CREATED"
    LOCATION_UPDATED = "LOCATION_UPDATED"
    MSP_INVOICE_PROPOSAL_STATUS = "MSP_INVOICE_PROPOSAL_STATUS"
    OPERATIONAL_STATUS = "OPERATIONAL_STATUS"
    PING = "PING"
    SESSION_START = "SESSION_START"
    SESSION_STOP = "SESSION_STOP"
    SESSION_UPDATE = "SESSION_UPDATE"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union
from ..models.tariff_post_dto_usage_type import TariffPostDtoUsageType

if TYPE_CHECKING:
    from ..models.private_emp_tariff_dto import PrivateEmpTariffDto
    from ..models.tariff_restriction import TariffRestriction


T = TypeVar("T", bound="TariffPostDto")


@_attrs_define
class TariffPostDto:
    """
    Attributes:
        name (Union[Unset, str]):
        currency (Union[Unset, str]):
        usage_type (Union[Unset, TariffPostDtoUsageType]):  Default: TariffPostDtoUsageType.SELL.
        private_emp_tariff (Union[Unset, PrivateEmpTariffDto]):
        start_tariff (Union[Unset, float]):
        price (Union[Unset, float]):
        parking_tariff (Union[Unset, float]):
        parking_step_size_in_minutes (Union[Unset, int]):
        parking_grace_period_in_minutes (Union[Unset, int]):
        parking_tariff_restrictions (Union[Unset, List['TariffRestriction']]):
        time_tariff (Union[Unset, float]):
        time_step_size_in_minutes (Union[Unset, int]):
        time_grace_period_in_minutes (Union[Unset, int]):
        external_reference (Union[Unset, str]):
    """

    name: Union[Unset, str] = UNSET
    currency: Union[Unset, str] = UNSET
    usage_type: Union[Unset, TariffPostDtoUsageType] = TariffPostDtoUsageType.SELL
    private_emp_tariff: Union[Unset, "PrivateEmpTariffDto"] = UNSET
    start_tariff: Union[Unset, float] = UNSET
    price: Union[Unset, float] = UNSET
    parking_tariff: Union[Unset, float] = UNSET
    parking_step_size_in_minutes: Union[Unset, int] = UNSET
    parking_grace_period_in_minutes: Union[Unset, int] = UNSET
    parking_tariff_restrictions: Union[Unset, List["TariffRestriction"]] = UNSET
    time_tariff: Union[Unset, float] = UNSET
    time_step_size_in_minutes: Union[Unset, int] = UNSET
    time_grace_period_in_minutes: Union[Unset, int] = UNSET
    external_reference: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name

        currency = self.currency

        usage_type: Union[Unset, str] = UNSET
        if not isinstance(self.usage_type, Unset):
            usage_type = self.usage_type.value

        private_emp_tariff: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.private_emp_tariff, Unset):
            private_emp_tariff = self.private_emp_tariff.to_dict()

        start_tariff = self.start_tariff

        price = self.price

        parking_tariff = self.parking_tariff

        parking_step_size_in_minutes = self.parking_step_size_in_minutes

        parking_grace_period_in_minutes = self.parking_grace_period_in_minutes

        parking_tariff_restrictions: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.parking_tariff_restrictions, Unset):
            parking_tariff_restrictions = []
            for (
                parking_tariff_restrictions_item_data
            ) in self.parking_tariff_restrictions:
                parking_tariff_restrictions_item = (
                    parking_tariff_restrictions_item_data.to_dict()
                )
                parking_tariff_restrictions.append(parking_tariff_restrictions_item)

        time_tariff = self.time_tariff

        time_step_size_in_minutes = self.time_step_size_in_minutes

        time_grace_period_in_minutes = self.time_grace_period_in_minutes

        external_reference = self.external_reference

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if name is not UNSET:
            field_dict["name"] = name
        if currency is not UNSET:
            field_dict["currency"] = currency
        if usage_type is not UNSET:
            field_dict["usageType"] = usage_type
        if private_emp_tariff is not UNSET:
            field_dict["privateEmpTariff"] = private_emp_tariff
        if start_tariff is not UNSET:
            field_dict["startTariff"] = start_tariff
        if price is not UNSET:
            field_dict["price"] = price
        if parking_tariff is not UNSET:
            field_dict["parkingTariff"] = parking_tariff
        if parking_step_size_in_minutes is not UNSET:
            field_dict["parkingStepSizeInMinutes"] = parking_step_size_in_minutes
        if parking_grace_period_in_minutes is not UNSET:
            field_dict["parkingGracePeriodInMinutes"] = parking_grace_period_in_minutes
        if parking_tariff_restrictions is not UNSET:
            field_dict["parkingTariffRestrictions"] = parking_tariff_restrictions
        if time_tariff is not UNSET:
            field_dict["timeTariff"] = time_tariff
        if time_step_size_in_minutes is not UNSET:
            field_dict["timeStepSizeInMinutes"] = time_step_size_in_minutes
        if time_grace_period_in_minutes is not UNSET:
            field_dict["timeGracePeriodInMinutes"] = time_grace_period_in_minutes
        if external_reference is not UNSET:
            field_dict["externalReference"] = external_reference

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.private_emp_tariff_dto import PrivateEmpTariffDto
        from ..models.tariff_restriction import TariffRestriction

        d = src_dict.copy()
        name = d.pop("name", UNSET)

        currency = d.pop("currency", UNSET)

        _usage_type = d.pop("usageType", UNSET)
        usage_type: Union[Unset, TariffPostDtoUsageType]
        if isinstance(_usage_type, Unset) or _usage_type is None:
            usage_type = UNSET
        else:
            usage_type = TariffPostDtoUsageType(_usage_type)

        _private_emp_tariff = d.pop("privateEmpTariff", UNSET)
        private_emp_tariff: Union[Unset, PrivateEmpTariffDto]
        if isinstance(_private_emp_tariff, Unset) or _private_emp_tariff is None:
            private_emp_tariff = UNSET
        else:
            private_emp_tariff = PrivateEmpTariffDto.from_dict(_private_emp_tariff)

        start_tariff = d.pop("startTariff", UNSET)

        price = d.pop("price", UNSET)

        parking_tariff = d.pop("parkingTariff", UNSET)

        parking_step_size_in_minutes = d.pop("parkingStepSizeInMinutes", UNSET)

        parking_grace_period_in_minutes = d.pop("parkingGracePeriodInMinutes", UNSET)

        parking_tariff_restrictions = []
        _parking_tariff_restrictions = d.pop("parkingTariffRestrictions", UNSET)
        for parking_tariff_restrictions_item_data in _parking_tariff_restrictions or []:
            parking_tariff_restrictions_item = TariffRestriction.from_dict(
                parking_tariff_restrictions_item_data
            )

            parking_tariff_restrictions.append(parking_tariff_restrictions_item)

        time_tariff = d.pop("timeTariff", UNSET)

        time_step_size_in_minutes = d.pop("timeStepSizeInMinutes", UNSET)

        time_grace_period_in_minutes = d.pop("timeGracePeriodInMinutes", UNSET)

        external_reference = d.pop("externalReference", UNSET)

        tariff_post_dto = cls(
            name=name,
            currency=currency,
            usage_type=usage_type,
            private_emp_tariff=private_emp_tariff,
            start_tariff=start_tariff,
            price=price,
            parking_tariff=parking_tariff,
            parking_step_size_in_minutes=parking_step_size_in_minutes,
            parking_grace_period_in_minutes=parking_grace_period_in_minutes,
            parking_tariff_restrictions=parking_tariff_restrictions,
            time_tariff=time_tariff,
            time_step_size_in_minutes=time_step_size_in_minutes,
            time_grace_period_in_minutes=time_grace_period_in_minutes,
            external_reference=external_reference,
        )

        tariff_post_dto.additional_properties = d
        return tariff_post_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class CdrStartedByInfoDtoAuthorizationState(str, Enum):
    APPROVEDBYALLOWANY = "ApprovedByAllowAny"
    APPROVEDBYAUTHREQUEST = "ApprovedByAuthRequest"
    APPROVEDBYLOCALTOKEN = "ApprovedByLocalToken"
    APPROVEDBYREMOTE = "ApprovedByRemote"
    APPROVEDBYROAMINGPARTNER = "ApprovedByRoamingPartner"
    ERRORINVALIDLOCALTOKEN = "ErrorInvalidLocalToken"
    ERRORNOLOCALORREMOTEAPPROVAL = "ErrorNoLocalOrRemoteApproval"
    ERRORNOROAMINGAPPROVAL = "ErrorNoRoamingApproval"
    ERRORUNKOWNREMOTETOKEN = "ErrorUnkownRemoteToken"
    WAITINGFORREMOTESTOP = "WaitingForRemoteStop"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union


T = TypeVar("T", bound="StringSegment")


@_attrs_define
class StringSegment:
    """
    Attributes:
        buffer (Union[Unset, str]):
        offset (Union[Unset, int]):
        length (Union[Unset, int]):
        value (Union[Unset, str]):
        has_value (Union[Unset, bool]):
    """

    buffer: Union[Unset, str] = UNSET
    offset: Union[Unset, int] = UNSET
    length: Union[Unset, int] = UNSET
    value: Union[Unset, str] = UNSET
    has_value: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        buffer = self.buffer

        offset = self.offset

        length = self.length

        value = self.value

        has_value = self.has_value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if buffer is not UNSET:
            field_dict["buffer"] = buffer
        if offset is not UNSET:
            field_dict["offset"] = offset
        if length is not UNSET:
            field_dict["length"] = length
        if value is not UNSET:
            field_dict["value"] = value
        if has_value is not UNSET:
            field_dict["hasValue"] = has_value

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        buffer = d.pop("buffer", UNSET)

        offset = d.pop("offset", UNSET)

        length = d.pop("length", UNSET)

        value = d.pop("value", UNSET)

        has_value = d.pop("hasValue", UNSET)

        string_segment = cls(
            buffer=buffer,
            offset=offset,
            length=length,
            value=value,
            has_value=has_value,
        )

        string_segment.additional_properties = d
        return string_segment

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import asyncio
import json
from datetime import datetime
from typing import List
from urllib.parse import parse_qs, urlsplit

from longship.errors import ChargepointNotFoundError, CompositeScheduleNotFoundError
from longship_api_client import Client
from longship_api_client.api.chargepoint_status import (
    chargepoint_status_get,
    get_all_chargepointstatus,
)
from longship_api_client.api.chargepoints import (
    chargepoint_get,
    get_all_chargepointmessages,
)
from longship_api_client.api.commands import (
    send_get_composite_schedule_request,
    send_set_charging_profile_request,
)
from longship_api_client.api.sessions import get_all_sessions
from longship_api_client.models import (
    GetCompositeScheduleRequest,
    SetChargingProfileRequest,
)
from longship_api_client.models.chargepoint_dto import ChargepointDto
from longship_api_client.models.chargepoint_status_dto import ChargepointStatusDto
from longship_api_client.models.charging_schedule import ChargingSchedule
from longship_api_client.models.charging_schedule_charging_rate_unit import (
    ChargingScheduleChargingRateUnit,
)
from longship_api_client.models.charging_schedule_period import ChargingSchedulePeriod
from longship_api_client.models.cs_charging_profiles import CsChargingProfiles
from longship_api_client.models.cs_charging_profiles_charging_profile_kind import (
    CsChargingProfilesChargingProfileKind,
)
from longship_api_client.models.cs_charging_profiles_charging_profile_purpose import (
    CsChargingProfilesChargingProfilePurpose,
)
from longship_api_client.models.get_composite_schedule_request_charging_rate_unit import (
    GetCompositeScheduleRequestChargingRateUnit,
)
from longship_api_client.models.message_log_dto import MessageLogDto
from longship_api_client.models.session_dto import SessionDto


class Longship:
    instance = None

    def __new__(cls, *args, **kwargs):
        if not isinstance(cls.instance, cls):
            cls.instance = object.__new__(cls)
        return cls.instance

    def __init__(self, url: str, apiKey: str, ocpKey: str) -> None:
        self._client = Client(base_url=url).with_headers(
            {"x-api-key": apiKey, "Ocp-Apim-Subscription-Key": ocpKey}
        )

    async def get_composite_schedule(
        self,
        chargepoint_id: str,
        connector_id=0,
        duration=3600,
        unit=GetCompositeScheduleRequestChargingRateUnit.W,
    ) -> ChargingSchedule:
        request = GetCompositeScheduleRequest(
            connector_id=connector_id, duration=duration, charging_rate_unit=unit
        )
        response = await send_get_composite_schedule_request.asyncio_detailed(
            id=chargepoint_id, json_body=request, client=self._client
        )
        if response.status_code == 404:
            raise ChargepointNotFoundError

        query = urlsplit(response.headers["location"]).query
        params = parse_qs(query)
        message_id = params["messageId"]
        await asyncio.sleep(1)

        response = await self.get_messages(
            chargepoint_id, response_only=True, message_id=message_id
        )
        if response.status_code != 200:
            raise Exception("Failed to load the messages")
        content = json.loads(response.content)
        if len(content) == 0:
            # TODO: Retry
            raise Exception("No response from the charger yet")
        payload = json.loads(content[0]["payload"])

        try:
            schedule = ChargingSchedule.from_dict(payload[2]["chargingSchedule"])
            return schedule
        except KeyError:
            raise CompositeScheduleNotFoundError

    def _create_charging_schedule(
        self, max_power: int, min_power: int = 1000, number_of_phases: int = 3
    ):
        period = ChargingSchedulePeriod(
            start_period=0, limit=max_power, number_phases=number_of_phases
        )
        schedule = ChargingSchedule(
            start_schedule=datetime.fromisoformat("2000-01-01T00:00:00"),
            charging_rate_unit=ChargingScheduleChargingRateUnit.W,
            min_charging_rate=min_power,
            charging_schedule_period=[period],
        )
        return schedule

    async def set_charge_point_max_power(
        self,
        chargepoint_id: str,
        max_power: int,
        min_power: int = 1000,
        number_of_phases: int = 3,
    ):
        schedule = self._create_charging_schedule(
            max_power, min_power, number_of_phases
        )
        profiles = CsChargingProfiles(
            charging_profile_id=1,
            stack_level=1,
            charging_profile_purpose=CsChargingProfilesChargingProfilePurpose.CHARGEPOINTMAXPROFILE,
            charging_profile_kind=CsChargingProfilesChargingProfileKind.ABSOLUTE,
            charging_schedule=schedule,
        )
        request = SetChargingProfileRequest(
            connector_id=0, cs_charging_profiles=profiles
        )
        return await send_set_charging_profile_request.asyncio_detailed(
            id=chargepoint_id, json_body=request, client=self._client
        )

    async def set_transaction_max_power(
        self,
        chargepoint_id: str,
        connector_id: int,
        transaction_id: int,
        max_power: int,
        min_power: int = 1000,
        number_of_phases: int = 3,
    ):
        schedule = self._create_charging_schedule(
            max_power, min_power, number_of_phases
        )
        profiles = CsChargingProfiles(
            charging_profile_id=2,
            stack_level=2,
            transaction_id=transaction_id,
            charging_profile_purpose=CsChargingProfilesChargingProfilePurpose.TXPROFILE,
            charging_profile_kind=CsChargingProfilesChargingProfileKind.ABSOLUTE,
            charging_schedule=schedule,
        )
        request = SetChargingProfileRequest(
            connector_id=connector_id, cs_charging_profiles=profiles
        )
        return await send_set_charging_profile_request.asyncio_detailed(
            id=chargepoint_id, json_body=request, client=self._client
        )

    async def get_messages(
        self,
        chargepoint_id: str,
        response_only=False,
        message_id=None,
        message_types=None,
    ) -> List[MessageLogDto]:
        return await get_all_chargepointmessages.asyncio_detailed(
            id=chargepoint_id,
            response_only=response_only,
            message_id=message_id,
            message_types=message_types,
            client=self._client,
        )

    async def get_chargepoint(self, chargepoint_id: str) -> ChargepointDto:
        response = await chargepoint_get.asyncio_detailed(
            id=chargepoint_id, client=self._client
        )
        return response.parsed

    async def list_chargepoint_statuses(
        self, skip: int = None, take: int = None
    ) -> ChargepointStatusDto:
        response = await get_all_chargepointstatus.asyncio_detailed(
            skip=skip, take=take, client=self._client
        )
        return response.parsed

    async def get_chargepoint_status(self, chargepoint_id: str) -> ChargepointStatusDto:
        response = await chargepoint_status_get.asyncio_detailed(
            id=chargepoint_id, client=self._client
        )
        return response.parsed

    async def get_sessions(
        self, chargepoint_id=None, connector_number=None, running_only=True
    ) -> List[SessionDto]:
        response = await get_all_sessions.asyncio_detailed(
            chargepoint_id=chargepoint_id,
            connector_number=connector_number,
            running_only=running_only,
            client=self._client,
        )
        return response.parsed



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.session_threshold_value_dto_decimal_threshold_hit_outcome import (
    SessionThresholdValueDtoDecimalThresholdHitOutcome,
)
from ..models.session_threshold_value_dto_decimal_status import (
    SessionThresholdValueDtoDecimalStatus,
)
from typing import Union


T = TypeVar("T", bound="SessionThresholdValueDtoDecimal")


@_attrs_define
class SessionThresholdValueDtoDecimal:
    """
    Attributes:
        threshold_value (Union[Unset, float]):
        threshold_hit_outcome (Union[Unset, SessionThresholdValueDtoDecimalThresholdHitOutcome]):  Default:
            SessionThresholdValueDtoDecimalThresholdHitOutcome.SUSPICIOUS.
        status (Union[Unset, SessionThresholdValueDtoDecimalStatus]):  Default:
            SessionThresholdValueDtoDecimalStatus.PENDING.
        result (Union[Unset, str]):
        is_enabled (Union[Unset, bool]):
    """

    threshold_value: Union[Unset, float] = UNSET
    threshold_hit_outcome: Union[
        Unset, SessionThresholdValueDtoDecimalThresholdHitOutcome
    ] = SessionThresholdValueDtoDecimalThresholdHitOutcome.SUSPICIOUS
    status: Union[Unset, SessionThresholdValueDtoDecimalStatus] = (
        SessionThresholdValueDtoDecimalStatus.PENDING
    )
    result: Union[Unset, str] = UNSET
    is_enabled: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        threshold_value = self.threshold_value

        threshold_hit_outcome: Union[Unset, str] = UNSET
        if not isinstance(self.threshold_hit_outcome, Unset):
            threshold_hit_outcome = self.threshold_hit_outcome.value

        status: Union[Unset, str] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.value

        result = self.result

        is_enabled = self.is_enabled

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if threshold_value is not UNSET:
            field_dict["thresholdValue"] = threshold_value
        if threshold_hit_outcome is not UNSET:
            field_dict["thresholdHitOutcome"] = threshold_hit_outcome
        if status is not UNSET:
            field_dict["status"] = status
        if result is not UNSET:
            field_dict["result"] = result
        if is_enabled is not UNSET:
            field_dict["isEnabled"] = is_enabled

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        threshold_value = d.pop("thresholdValue", UNSET)

        _threshold_hit_outcome = d.pop("thresholdHitOutcome", UNSET)
        threshold_hit_outcome: Union[
            Unset, SessionThresholdValueDtoDecimalThresholdHitOutcome
        ]
        if isinstance(_threshold_hit_outcome, Unset) or _threshold_hit_outcome is None:
            threshold_hit_outcome = UNSET
        else:
            threshold_hit_outcome = SessionThresholdValueDtoDecimalThresholdHitOutcome(
                _threshold_hit_outcome
            )

        _status = d.pop("status", UNSET)
        status: Union[Unset, SessionThresholdValueDtoDecimalStatus]
        if isinstance(_status, Unset) or _status is None:
            status = UNSET
        else:
            status = SessionThresholdValueDtoDecimalStatus(_status)

        result = d.pop("result", UNSET)

        is_enabled = d.pop("isEnabled", UNSET)

        session_threshold_value_dto_decimal = cls(
            threshold_value=threshold_value,
            threshold_hit_outcome=threshold_hit_outcome,
            status=status,
            result=result,
            is_enabled=is_enabled,
        )

        session_threshold_value_dto_decimal.additional_properties = d
        return session_threshold_value_dto_decimal

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.authorization_result_dto_status import AuthorizationResultDtoStatus
from typing import Union
from ..models.authorization_result_dto_reason import AuthorizationResultDtoReason

if TYPE_CHECKING:
    from ..models.authorization_assertion_dto import AuthorizationAssertionDto


T = TypeVar("T", bound="AuthorizationResultDto")


@_attrs_define
class AuthorizationResultDto:
    """
    Attributes:
        assertion (Union[Unset, List['AuthorizationAssertionDto']]):
        status (Union[Unset, AuthorizationResultDtoStatus]):  Default: AuthorizationResultDtoStatus.ACCEPTED.
        reason (Union[Unset, AuthorizationResultDtoReason]):  Default: AuthorizationResultDtoReason.APPROVEDBYREMOTE.
        description (Union[Unset, str]):
    """

    assertion: Union[Unset, List["AuthorizationAssertionDto"]] = UNSET
    status: Union[Unset, AuthorizationResultDtoStatus] = (
        AuthorizationResultDtoStatus.ACCEPTED
    )
    reason: Union[Unset, AuthorizationResultDtoReason] = (
        AuthorizationResultDtoReason.APPROVEDBYREMOTE
    )
    description: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        assertion: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.assertion, Unset):
            assertion = []
            for assertion_item_data in self.assertion:
                assertion_item = assertion_item_data.to_dict()
                assertion.append(assertion_item)

        status: Union[Unset, str] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.value

        reason: Union[Unset, str] = UNSET
        if not isinstance(self.reason, Unset):
            reason = self.reason.value

        description = self.description

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if assertion is not UNSET:
            field_dict["assertion"] = assertion
        if status is not UNSET:
            field_dict["status"] = status
        if reason is not UNSET:
            field_dict["reason"] = reason
        if description is not UNSET:
            field_dict["description"] = description

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.authorization_assertion_dto import AuthorizationAssertionDto

        d = src_dict.copy()
        assertion = []
        _assertion = d.pop("assertion", UNSET)
        for assertion_item_data in _assertion or []:
            assertion_item = AuthorizationAssertionDto.from_dict(assertion_item_data)

            assertion.append(assertion_item)

        _status = d.pop("status", UNSET)
        status: Union[Unset, AuthorizationResultDtoStatus]
        if isinstance(_status, Unset) or _status is None:
            status = UNSET
        else:
            status = AuthorizationResultDtoStatus(_status)

        _reason = d.pop("reason", UNSET)
        reason: Union[Unset, AuthorizationResultDtoReason]
        if isinstance(_reason, Unset) or _reason is None:
            reason = UNSET
        else:
            reason = AuthorizationResultDtoReason(_reason)

        description = d.pop("description", UNSET)

        authorization_result_dto = cls(
            assertion=assertion,
            status=status,
            reason=reason,
            description=description,
        )

        authorization_result_dto.additional_properties = d
        return authorization_result_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class MessageLogDtoOcppMessageType(str, Enum):
    AUTHORIZE = "Authorize"
    BOOTNOTIFICATION = "BootNotification"
    CANCELRESERVATION = "CancelReservation"
    CHANGEAVAILABILITY = "ChangeAvailability"
    CHANGECONFIGURATION = "ChangeConfiguration"
    CLEARCACHE = "ClearCache"
    CLEARCHARCHINGPROFILE = "ClearCharchingProfile"
    DATATRANSFER = "DataTransfer"
    DIAGNOSTICSSTATUSNOTIFICATION = "DiagnosticsStatusNotification"
    FIRMWARESTATUSNOTIFICATION = "FirmwareStatusNotification"
    GETCOMPOSITESCHEDULE = "GetCompositeSchedule"
    GETCONFIGURATION = "GetConfiguration"
    GETDIAGNOSTICS = "GetDiagnostics"
    GETLOCALLIST = "GetLocalList"
    HEARTBEAT = "Heartbeat"
    METERVALUES = "MeterValues"
    REMOTESTARTTRANSACTION = "RemoteStartTransaction"
    REMOTESTOPTRANSACTION = "RemoteStopTransaction"
    RESERVENOW = "ReserveNow"
    RESET = "Reset"
    SENDLOCALLIST = "SendLocalList"
    SETCHARGINGPROFILE = "SetChargingProfile"
    STARTTRANSACTION = "StartTransaction"
    STATUSNOTIFICATION = "StatusNotification"
    STOPTRANSACTION = "StopTransaction"
    TRANSACTIONEVENT = "TransactionEvent"
    TRIGGERMESSAGE = "TriggerMessage"
    UNLOCKCONNECTOR = "UnlockConnector"
    UPDATEFIRMWARE = "UpdateFirmware"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union
from ..models.clear_charging_profile_request_charging_profile_purpose import (
    ClearChargingProfileRequestChargingProfilePurpose,
)


T = TypeVar("T", bound="ClearChargingProfileRequest")


@_attrs_define
class ClearChargingProfileRequest:
    """
    Attributes:
        id (Union[Unset, int]):
        connector_id (Union[Unset, int]):
        charging_profile_purpose (Union[Unset, ClearChargingProfileRequestChargingProfilePurpose]):  Default:
            ClearChargingProfileRequestChargingProfilePurpose.CHARGEPOINTMAXPROFILE.
        stack_level (Union[Unset, int]):
    """

    id: Union[Unset, int] = UNSET
    connector_id: Union[Unset, int] = UNSET
    charging_profile_purpose: Union[
        Unset, ClearChargingProfileRequestChargingProfilePurpose
    ] = ClearChargingProfileRequestChargingProfilePurpose.CHARGEPOINTMAXPROFILE
    stack_level: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        connector_id = self.connector_id

        charging_profile_purpose: Union[Unset, str] = UNSET
        if not isinstance(self.charging_profile_purpose, Unset):
            charging_profile_purpose = self.charging_profile_purpose.value

        stack_level = self.stack_level

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if connector_id is not UNSET:
            field_dict["connectorId"] = connector_id
        if charging_profile_purpose is not UNSET:
            field_dict["chargingProfilePurpose"] = charging_profile_purpose
        if stack_level is not UNSET:
            field_dict["stackLevel"] = stack_level

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", UNSET)

        connector_id = d.pop("connectorId", UNSET)

        _charging_profile_purpose = d.pop("chargingProfilePurpose", UNSET)
        charging_profile_purpose: Union[
            Unset, ClearChargingProfileRequestChargingProfilePurpose
        ]
        if isinstance(_charging_profile_purpose, Unset) or _charging_profile_purpose is None:
            charging_profile_purpose = UNSET
        else:
            charging_profile_purpose = (
                ClearChargingProfileRequestChargingProfilePurpose(
                    _charging_profile_purpose
                )
            )

        stack_level = d.pop("stackLevel", UNSET)

        clear_charging_profile_request = cls(
            id=id,
            connector_id=connector_id,
            charging_profile_purpose=charging_profile_purpose,
            stack_level=stack_level,
        )

        clear_charging_profile_request.additional_properties = d
        return clear_charging_profile_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class SessionThresholdValueDtoDecimalThresholdHitOutcome(str, Enum):
    REJECTED = "Rejected"
    SUSPICIOUS = "Suspicious"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from dateutil.parser import isoparse
import datetime
from typing import Union
from ..models.tariff_restriction_day_of_week import TariffRestrictionDayOfWeek


T = TypeVar("T", bound="TariffRestriction")


@_attrs_define
class TariffRestriction:
    """
    Attributes:
        start_time (Union[Unset, str]):
        end_time (Union[Unset, str]):
        start_date (Union[Unset, datetime.datetime]):
        end_date (Union[Unset, datetime.datetime]):
        day_of_week (Union[Unset, TariffRestrictionDayOfWeek]):  Default: TariffRestrictionDayOfWeek.VALUE_0.
    """

    start_time: Union[Unset, str] = UNSET
    end_time: Union[Unset, str] = UNSET
    start_date: Union[Unset, datetime.datetime] = UNSET
    end_date: Union[Unset, datetime.datetime] = UNSET
    day_of_week: Union[Unset, TariffRestrictionDayOfWeek] = (
        TariffRestrictionDayOfWeek.VALUE_0
    )
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        start_time = self.start_time

        end_time = self.end_time

        start_date: Union[Unset, str] = UNSET
        if not isinstance(self.start_date, Unset):
            start_date = self.start_date.isoformat()

        end_date: Union[Unset, str] = UNSET
        if not isinstance(self.end_date, Unset):
            end_date = self.end_date.isoformat()

        day_of_week: Union[Unset, int] = UNSET
        if not isinstance(self.day_of_week, Unset):
            day_of_week = self.day_of_week.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if start_time is not UNSET:
            field_dict["startTime"] = start_time
        if end_time is not UNSET:
            field_dict["endTime"] = end_time
        if start_date is not UNSET:
            field_dict["startDate"] = start_date
        if end_date is not UNSET:
            field_dict["endDate"] = end_date
        if day_of_week is not UNSET:
            field_dict["dayOfWeek"] = day_of_week

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        start_time = d.pop("startTime", UNSET)

        end_time = d.pop("endTime", UNSET)

        _start_date = d.pop("startDate", UNSET)
        start_date: Union[Unset, datetime.datetime]
        if isinstance(_start_date, Unset) or _start_date is None:
            start_date = UNSET
        else:
            start_date = isoparse(_start_date)

        _end_date = d.pop("endDate", UNSET)
        end_date: Union[Unset, datetime.datetime]
        if isinstance(_end_date, Unset) or _end_date is None:
            end_date = UNSET
        else:
            end_date = isoparse(_end_date)

        _day_of_week = d.pop("dayOfWeek", UNSET)
        day_of_week: Union[Unset, TariffRestrictionDayOfWeek]
        if isinstance(_day_of_week, Unset) or _day_of_week is None:
            day_of_week = UNSET
        else:
            day_of_week = TariffRestrictionDayOfWeek(_day_of_week)

        tariff_restriction = cls(
            start_time=start_time,
            end_time=end_time,
            start_date=start_date,
            end_date=end_date,
            day_of_week=day_of_week,
        )

        tariff_restriction.additional_properties = d
        return tariff_restriction

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class MessageLogDtoWampMessageType(str, Enum):
    ERROR = "Error"
    REQUEST = "Request"
    RESPONSE = "Response"
    UNKNOWN = "Unknown"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class SessionThresholdsDtoThresholdsHitItem(str, Enum):
    CHECKCHARGINGSPEED = "CheckChargingSpeed"
    CHECKSESSIONINFUTURE = "CheckSessionInFuture"
    MAXKWHAC = "MaxKwhAc"
    MAXKWHDC = "MaxKwhDc"
    MAXSESSIONAGEINDAYS = "MaxSessionAgeInDays"
    MINDURATIONAC = "MinDurationAc"
    MINDURATIONDC = "MinDurationDc"
    MINKWHAC = "MinKwhAc"
    MINKWHDC = "MinKwhDc"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class AuthorizationAssertionDtoAuthScenarioType(str, Enum):
    CHARGERALLOWANY = "ChargerAllowAny"
    EXISTINGHUBJECTAPPROVAL = "ExistingHubjectApproval"
    HUBJECTFASTAPPROVAL = "HubjectFastApproval"
    LOCALTOKEN = "LocalToken"
    MSPBLOCKED = "MspBlocked"
    NEWTOKENVALIDATIONREQUEST = "NewTokenValidationRequest"
    NOAPPROVALS = "NoApprovals"
    NONE = "None"
    RECENTPLUGANDCHARGEAPPROVAL = "RecentPlugAndChargeApproval"
    REIMBURSEMENT = "Reimbursement"
    REMOTESTARTTOKEN = "RemoteStartToken"
    WHITELISTEDTOKEN = "WhitelistedToken"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union


T = TypeVar("T", bound="AuthorizationTenantContextDto")


@_attrs_define
class AuthorizationTenantContextDto:
    """
    Attributes:
        hubject_priority (Union[Unset, int]):
        hubject_enabled (Union[Unset, bool]):
        hubject_fast_approval (Union[Unset, bool]):
    """

    hubject_priority: Union[Unset, int] = UNSET
    hubject_enabled: Union[Unset, bool] = UNSET
    hubject_fast_approval: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        hubject_priority = self.hubject_priority

        hubject_enabled = self.hubject_enabled

        hubject_fast_approval = self.hubject_fast_approval

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if hubject_priority is not UNSET:
            field_dict["hubjectPriority"] = hubject_priority
        if hubject_enabled is not UNSET:
            field_dict["hubjectEnabled"] = hubject_enabled
        if hubject_fast_approval is not UNSET:
            field_dict["hubjectFastApproval"] = hubject_fast_approval

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        hubject_priority = d.pop("hubjectPriority", UNSET)

        hubject_enabled = d.pop("hubjectEnabled", UNSET)

        hubject_fast_approval = d.pop("hubjectFastApproval", UNSET)

        authorization_tenant_context_dto = cls(
            hubject_priority=hubject_priority,
            hubject_enabled=hubject_enabled,
            hubject_fast_approval=hubject_fast_approval,
        )

        authorization_tenant_context_dto.additional_properties = d
        return authorization_tenant_context_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from dateutil.parser import isoparse
from ..models.status_schedule_dto_status import StatusScheduleDtoStatus
import datetime
from typing import Union


T = TypeVar("T", bound="StatusScheduleDto")


@_attrs_define
class StatusScheduleDto:
    """
    Attributes:
        period_begin (Union[Unset, datetime.datetime]):
        period_end (Union[Unset, datetime.datetime]):
        status (Union[Unset, StatusScheduleDtoStatus]):  Default: StatusScheduleDtoStatus.AVAILABLE.
    """

    period_begin: Union[Unset, datetime.datetime] = UNSET
    period_end: Union[Unset, datetime.datetime] = UNSET
    status: Union[Unset, StatusScheduleDtoStatus] = StatusScheduleDtoStatus.AVAILABLE
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        period_begin: Union[Unset, str] = UNSET
        if not isinstance(self.period_begin, Unset):
            period_begin = self.period_begin.isoformat()

        period_end: Union[Unset, str] = UNSET
        if not isinstance(self.period_end, Unset):
            period_end = self.period_end.isoformat()

        status: Union[Unset, str] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if period_begin is not UNSET:
            field_dict["period_begin"] = period_begin
        if period_end is not UNSET:
            field_dict["period_end"] = period_end
        if status is not UNSET:
            field_dict["status"] = status

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _period_begin = d.pop("period_begin", UNSET)
        period_begin: Union[Unset, datetime.datetime]
        if isinstance(_period_begin, Unset) or _period_begin is None:
            period_begin = UNSET
        else:
            period_begin = isoparse(_period_begin)

        _period_end = d.pop("period_end", UNSET)
        period_end: Union[Unset, datetime.datetime]
        if isinstance(_period_end, Unset) or _period_end is None:
            period_end = UNSET
        else:
            period_end = isoparse(_period_end)

        _status = d.pop("status", UNSET)
        status: Union[Unset, StatusScheduleDtoStatus]
        if isinstance(_status, Unset) or _status is None:
            status = UNSET
        else:
            status = StatusScheduleDtoStatus(_status)

        status_schedule_dto = cls(
            period_begin=period_begin,
            period_end=period_end,
            status=status,
        )

        status_schedule_dto.additional_properties = d
        return status_schedule_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union

if TYPE_CHECKING:
    from ..models.id_tag_info import IdTagInfo


T = TypeVar("T", bound="AuthorizationData")


@_attrs_define
class AuthorizationData:
    """
    Attributes:
        id_tag (str):
        id_tag_info (Union[Unset, IdTagInfo]):
    """

    id_tag: str
    id_tag_info: Union[Unset, "IdTagInfo"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id_tag = self.id_tag

        id_tag_info: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.id_tag_info, Unset):
            id_tag_info = self.id_tag_info.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "idTag": id_tag,
            }
        )
        if id_tag_info is not UNSET:
            field_dict["idTagInfo"] = id_tag_info

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.id_tag_info import IdTagInfo

        d = src_dict.copy()
        id_tag = d.pop("idTag")

        _id_tag_info = d.pop("idTagInfo", UNSET)
        id_tag_info: Union[Unset, IdTagInfo]
        if isinstance(_id_tag_info, Unset) or _id_tag_info is None:
            id_tag_info = UNSET
        else:
            id_tag_info = IdTagInfo.from_dict(_id_tag_info)

        authorization_data = cls(
            id_tag=id_tag,
            id_tag_info=id_tag_info,
        )

        authorization_data.additional_properties = d
        return authorization_data

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class ClearChargingProfileRequestChargingProfilePurpose(str, Enum):
    CHARGEPOINTMAXPROFILE = "ChargePointMaxProfile"
    TXDEFAULTPROFILE = "TxDefaultProfile"
    TXPROFILE = "TxProfile"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.publish_token_type_dto_type import PublishTokenTypeDtoType
from typing import Union


T = TypeVar("T", bound="PublishTokenTypeDto")


@_attrs_define
class PublishTokenTypeDto:
    """
    Attributes:
        uid (Union[Unset, str]):
        type (Union[Unset, PublishTokenTypeDtoType]):  Default: PublishTokenTypeDtoType.RFID.
        visual_number (Union[Unset, str]):
        issuer (Union[Unset, str]):
        group_id (Union[Unset, str]):
    """

    uid: Union[Unset, str] = UNSET
    type: Union[Unset, PublishTokenTypeDtoType] = PublishTokenTypeDtoType.RFID
    visual_number: Union[Unset, str] = UNSET
    issuer: Union[Unset, str] = UNSET
    group_id: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        uid = self.uid

        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value

        visual_number = self.visual_number

        issuer = self.issuer

        group_id = self.group_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if uid is not UNSET:
            field_dict["uid"] = uid
        if type is not UNSET:
            field_dict["type"] = type
        if visual_number is not UNSET:
            field_dict["visual_number"] = visual_number
        if issuer is not UNSET:
            field_dict["issuer"] = issuer
        if group_id is not UNSET:
            field_dict["group_id"] = group_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        uid = d.pop("uid", UNSET)

        _type = d.pop("type", UNSET)
        type: Union[Unset, PublishTokenTypeDtoType]
        if isinstance(_type, Unset) or _type is None:
            type = UNSET
        else:
            type = PublishTokenTypeDtoType(_type)

        visual_number = d.pop("visual_number", UNSET)

        issuer = d.pop("issuer", UNSET)

        group_id = d.pop("group_id", UNSET)

        publish_token_type_dto = cls(
            uid=uid,
            type=type,
            visual_number=visual_number,
            issuer=issuer,
            group_id=group_id,
        )

        publish_token_type_dto.additional_properties = d
        return publish_token_type_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from dateutil.parser import isoparse
import datetime
from typing import Union


T = TypeVar("T", bound="InterchangeFormatCdr")


@_attrs_define
class InterchangeFormatCdr:
    """
    Attributes:
        cdr_id (Union[Unset, str]):
        start_date_time (Union[Unset, datetime.datetime]):
        end_date_time (Union[Unset, datetime.datetime]):
        duration (Union[Unset, str]):
        volume (Union[Unset, float]):
        charge_point_address (Union[Unset, str]):
        charge_point_zip (Union[Unset, str]):
        charge_point_city (Union[Unset, str]):
        charge_point_country (Union[Unset, str]):
        charge_point_type (Union[Unset, str]):
        product_type (Union[Unset, str]):
        tariff_type (Union[Unset, str]):
        authentication_id (Union[Unset, str]):
        contract_id (Union[Unset, str]):
        meter_id (Union[Unset, str]):
        obis_code (Union[Unset, str]):
        charge_point_id (Union[Unset, str]):
        service_provider_id (Union[Unset, str]):
        infra_provider_id (Union[Unset, str]):
        calculated_cost (Union[Unset, float]):
    """

    cdr_id: Union[Unset, str] = UNSET
    start_date_time: Union[Unset, datetime.datetime] = UNSET
    end_date_time: Union[Unset, datetime.datetime] = UNSET
    duration: Union[Unset, str] = UNSET
    volume: Union[Unset, float] = UNSET
    charge_point_address: Union[Unset, str] = UNSET
    charge_point_zip: Union[Unset, str] = UNSET
    charge_point_city: Union[Unset, str] = UNSET
    charge_point_country: Union[Unset, str] = UNSET
    charge_point_type: Union[Unset, str] = UNSET
    product_type: Union[Unset, str] = UNSET
    tariff_type: Union[Unset, str] = UNSET
    authentication_id: Union[Unset, str] = UNSET
    contract_id: Union[Unset, str] = UNSET
    meter_id: Union[Unset, str] = UNSET
    obis_code: Union[Unset, str] = UNSET
    charge_point_id: Union[Unset, str] = UNSET
    service_provider_id: Union[Unset, str] = UNSET
    infra_provider_id: Union[Unset, str] = UNSET
    calculated_cost: Union[Unset, float] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        cdr_id = self.cdr_id

        start_date_time: Union[Unset, str] = UNSET
        if not isinstance(self.start_date_time, Unset):
            start_date_time = self.start_date_time.isoformat()

        end_date_time: Union[Unset, str] = UNSET
        if not isinstance(self.end_date_time, Unset):
            end_date_time = self.end_date_time.isoformat()

        duration = self.duration

        volume = self.volume

        charge_point_address = self.charge_point_address

        charge_point_zip = self.charge_point_zip

        charge_point_city = self.charge_point_city

        charge_point_country = self.charge_point_country

        charge_point_type = self.charge_point_type

        product_type = self.product_type

        tariff_type = self.tariff_type

        authentication_id = self.authentication_id

        contract_id = self.contract_id

        meter_id = self.meter_id

        obis_code = self.obis_code

        charge_point_id = self.charge_point_id

        service_provider_id = self.service_provider_id

        infra_provider_id = self.infra_provider_id

        calculated_cost = self.calculated_cost

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if cdr_id is not UNSET:
            field_dict["cdrId"] = cdr_id
        if start_date_time is not UNSET:
            field_dict["startDateTime"] = start_date_time
        if end_date_time is not UNSET:
            field_dict["endDateTime"] = end_date_time
        if duration is not UNSET:
            field_dict["duration"] = duration
        if volume is not UNSET:
            field_dict["volume"] = volume
        if charge_point_address is not UNSET:
            field_dict["chargePointAddress"] = charge_point_address
        if charge_point_zip is not UNSET:
            field_dict["chargePointZip"] = charge_point_zip
        if charge_point_city is not UNSET:
            field_dict["chargePointCity"] = charge_point_city
        if charge_point_country is not UNSET:
            field_dict["chargePointCountry"] = charge_point_country
        if charge_point_type is not UNSET:
            field_dict["chargePointType"] = charge_point_type
        if product_type is not UNSET:
            field_dict["productType"] = product_type
        if tariff_type is not UNSET:
            field_dict["tariffType"] = tariff_type
        if authentication_id is not UNSET:
            field_dict["authenticationId"] = authentication_id
        if contract_id is not UNSET:
            field_dict["contractId"] = contract_id
        if meter_id is not UNSET:
            field_dict["meterId"] = meter_id
        if obis_code is not UNSET:
            field_dict["obisCode"] = obis_code
        if charge_point_id is not UNSET:
            field_dict["chargePointId"] = charge_point_id
        if service_provider_id is not UNSET:
            field_dict["serviceProviderId"] = service_provider_id
        if infra_provider_id is not UNSET:
            field_dict["infraProviderId"] = infra_provider_id
        if calculated_cost is not UNSET:
            field_dict["calculatedCost"] = calculated_cost

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        cdr_id = d.pop("cdrId", UNSET)

        _start_date_time = d.pop("startDateTime", UNSET)
        start_date_time: Union[Unset, datetime.datetime]
        if isinstance(_start_date_time, Unset) or _start_date_time is None:
            start_date_time = UNSET
        else:
            start_date_time = isoparse(_start_date_time)

        _end_date_time = d.pop("endDateTime", UNSET)
        end_date_time: Union[Unset, datetime.datetime]
        if isinstance(_end_date_time, Unset) or _end_date_time is None:
            end_date_time = UNSET
        else:
            end_date_time = isoparse(_end_date_time)

        duration = d.pop("duration", UNSET)

        volume = d.pop("volume", UNSET)

        charge_point_address = d.pop("chargePointAddress", UNSET)

        charge_point_zip = d.pop("chargePointZip", UNSET)

        charge_point_city = d.pop("chargePointCity", UNSET)

        charge_point_country = d.pop("chargePointCountry", UNSET)

        charge_point_type = d.pop("chargePointType", UNSET)

        product_type = d.pop("productType", UNSET)

        tariff_type = d.pop("tariffType", UNSET)

        authentication_id = d.pop("authenticationId", UNSET)

        contract_id = d.pop("contractId", UNSET)

        meter_id = d.pop("meterId", UNSET)

        obis_code = d.pop("obisCode", UNSET)

        charge_point_id = d.pop("chargePointId", UNSET)

        service_provider_id = d.pop("serviceProviderId", UNSET)

        infra_provider_id = d.pop("infraProviderId", UNSET)

        calculated_cost = d.pop("calculatedCost", UNSET)

        interchange_format_cdr = cls(
            cdr_id=cdr_id,
            start_date_time=start_date_time,
            end_date_time=end_date_time,
            duration=duration,
            volume=volume,
            charge_point_address=charge_point_address,
            charge_point_zip=charge_point_zip,
            charge_point_city=charge_point_city,
            charge_point_country=charge_point_country,
            charge_point_type=charge_point_type,
            product_type=product_type,
            tariff_type=tariff_type,
            authentication_id=authentication_id,
            contract_id=contract_id,
            meter_id=meter_id,
            obis_code=obis_code,
            charge_point_id=charge_point_id,
            service_provider_id=service_provider_id,
            infra_provider_id=infra_provider_id,
            calculated_cost=calculated_cost,
        )

        interchange_format_cdr.additional_properties = d
        return interchange_format_cdr

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from dateutil.parser import isoparse
import datetime
from typing import Union
from ..models.charge_point_authorize_get_dto_authorization_request_type import (
    ChargePointAuthorizeGetDtoAuthorizationRequestType,
)

if TYPE_CHECKING:
    from ..models.authorization_result_dto import AuthorizationResultDto
    from ..models.token_info_dto import TokenInfoDto
    from ..models.authorization_context_details_dto import (
        AuthorizationContextDetailsDto,
    )


T = TypeVar("T", bound="ChargePointAuthorizeGetDto")


@_attrs_define
class ChargePointAuthorizeGetDto:
    """
    Attributes:
        id (Union[Unset, str]):
        message_id (Union[Unset, str]):
        created (Union[Unset, datetime.datetime]):
        token_info (Union[Unset, TokenInfoDto]):
        authorization_request_type (Union[Unset, ChargePointAuthorizeGetDtoAuthorizationRequestType]):  Default:
            ChargePointAuthorizeGetDtoAuthorizationRequestType.AUTHORIZE.
        authorization_result (Union[Unset, AuthorizationResultDto]):
        context (Union[Unset, AuthorizationContextDetailsDto]):
    """

    id: Union[Unset, str] = UNSET
    message_id: Union[Unset, str] = UNSET
    created: Union[Unset, datetime.datetime] = UNSET
    token_info: Union[Unset, "TokenInfoDto"] = UNSET
    authorization_request_type: Union[
        Unset, ChargePointAuthorizeGetDtoAuthorizationRequestType
    ] = ChargePointAuthorizeGetDtoAuthorizationRequestType.AUTHORIZE
    authorization_result: Union[Unset, "AuthorizationResultDto"] = UNSET
    context: Union[Unset, "AuthorizationContextDetailsDto"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        message_id = self.message_id

        created: Union[Unset, str] = UNSET
        if not isinstance(self.created, Unset):
            created = self.created.isoformat()

        token_info: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.token_info, Unset):
            token_info = self.token_info.to_dict()

        authorization_request_type: Union[Unset, str] = UNSET
        if not isinstance(self.authorization_request_type, Unset):
            authorization_request_type = self.authorization_request_type.value

        authorization_result: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.authorization_result, Unset):
            authorization_result = self.authorization_result.to_dict()

        context: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.context, Unset):
            context = self.context.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if message_id is not UNSET:
            field_dict["messageId"] = message_id
        if created is not UNSET:
            field_dict["created"] = created
        if token_info is not UNSET:
            field_dict["tokenInfo"] = token_info
        if authorization_request_type is not UNSET:
            field_dict["authorizationRequestType"] = authorization_request_type
        if authorization_result is not UNSET:
            field_dict["authorizationResult"] = authorization_result
        if context is not UNSET:
            field_dict["context"] = context

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.authorization_result_dto import AuthorizationResultDto
        from ..models.token_info_dto import TokenInfoDto
        from ..models.authorization_context_details_dto import (
            AuthorizationContextDetailsDto,
        )

        d = src_dict.copy()
        id = d.pop("id", UNSET)

        message_id = d.pop("messageId", UNSET)

        _created = d.pop("created", UNSET)
        created: Union[Unset, datetime.datetime]
        if isinstance(_created, Unset) or _created is None:
            created = UNSET
        else:
            created = isoparse(_created)

        _token_info = d.pop("tokenInfo", UNSET)
        token_info: Union[Unset, TokenInfoDto]
        if isinstance(_token_info, Unset) or _token_info is None:
            token_info = UNSET
        else:
            token_info = TokenInfoDto.from_dict(_token_info)

        _authorization_request_type = d.pop("authorizationRequestType", UNSET)
        authorization_request_type: Union[
            Unset, ChargePointAuthorizeGetDtoAuthorizationRequestType
        ]
        if isinstance(_authorization_request_type, Unset) or _authorization_request_type is None:
            authorization_request_type = UNSET
        else:
            authorization_request_type = (
                ChargePointAuthorizeGetDtoAuthorizationRequestType(
                    _authorization_request_type
                )
            )

        _authorization_result = d.pop("authorizationResult", UNSET)
        authorization_result: Union[Unset, AuthorizationResultDto]
        if isinstance(_authorization_result, Unset) or _authorization_result is None:
            authorization_result = UNSET
        else:
            authorization_result = AuthorizationResultDto.from_dict(
                _authorization_result
            )

        _context = d.pop("context", UNSET)
        context: Union[Unset, AuthorizationContextDetailsDto]
        if isinstance(_context, Unset) or _context is None:
            context = UNSET
        else:
            context = AuthorizationContextDetailsDto.from_dict(_context)

        charge_point_authorize_get_dto = cls(
            id=id,
            message_id=message_id,
            created=created,
            token_info=token_info,
            authorization_request_type=authorization_request_type,
            authorization_result=authorization_result,
            context=context,
        )

        charge_point_authorize_get_dto.additional_properties = d
        return charge_point_authorize_get_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class GetAllReimbursementcdrsOrderBy(str, Enum):
    ENDDATETIME = "enddatetime"
    STARTDATETIME = "startdatetime"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from dateutil.parser import isoparse
import datetime
from typing import Union


T = TypeVar("T", bound="UpdateFirmwareRequest")


@_attrs_define
class UpdateFirmwareRequest:
    """
    Attributes:
        location (str):
        retrieve_date (datetime.datetime):
        retries (Union[Unset, int]):
        retry_interval (Union[Unset, int]):
    """

    location: str
    retrieve_date: datetime.datetime
    retries: Union[Unset, int] = UNSET
    retry_interval: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        location = self.location

        retrieve_date = self.retrieve_date.isoformat()

        retries = self.retries

        retry_interval = self.retry_interval

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "location": location,
                "retrieveDate": retrieve_date,
            }
        )
        if retries is not UNSET:
            field_dict["retries"] = retries
        if retry_interval is not UNSET:
            field_dict["retryInterval"] = retry_interval

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        location = d.pop("location")

        retrieve_date = isoparse(d.pop("retrieveDate"))

        retries = d.pop("retries", UNSET)

        retry_interval = d.pop("retryInterval", UNSET)

        update_firmware_request = cls(
            location=location,
            retrieve_date=retrieve_date,
            retries=retries,
            retry_interval=retry_interval,
        )

        update_firmware_request.additional_properties = d
        return update_firmware_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from dateutil.parser import isoparse
import datetime
from ..models.connector_dto_format import ConnectorDtoFormat
from ..models.connector_dto_power_type import ConnectorDtoPowerType
from typing import Union
from ..models.connector_dto_standard import ConnectorDtoStandard


T = TypeVar("T", bound="ConnectorDto")


@_attrs_define
class ConnectorDto:
    """
    Attributes:
        id (Union[Unset, str]):
        standard (Union[Unset, ConnectorDtoStandard]):  Default: ConnectorDtoStandard.CHADEMO.
        format_ (Union[Unset, ConnectorDtoFormat]):  Default: ConnectorDtoFormat.SOCKET.
        power_type (Union[Unset, ConnectorDtoPowerType]):  Default: ConnectorDtoPowerType.AC_1_PHASE.
        max_voltage (Union[Unset, int]):
        max_amperage (Union[Unset, int]):
        max_electric_power (Union[Unset, int]):
        calc_max_electric_power (Union[Unset, bool]):
        last_updated (Union[Unset, datetime.datetime]):
    """

    id: Union[Unset, str] = UNSET
    standard: Union[Unset, ConnectorDtoStandard] = ConnectorDtoStandard.CHADEMO
    format_: Union[Unset, ConnectorDtoFormat] = ConnectorDtoFormat.SOCKET
    power_type: Union[Unset, ConnectorDtoPowerType] = ConnectorDtoPowerType.AC_1_PHASE
    max_voltage: Union[Unset, int] = UNSET
    max_amperage: Union[Unset, int] = UNSET
    max_electric_power: Union[Unset, int] = UNSET
    calc_max_electric_power: Union[Unset, bool] = UNSET
    last_updated: Union[Unset, datetime.datetime] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        standard: Union[Unset, str] = UNSET
        if not isinstance(self.standard, Unset):
            standard = self.standard.value

        format_: Union[Unset, str] = UNSET
        if not isinstance(self.format_, Unset):
            format_ = self.format_.value

        power_type: Union[Unset, str] = UNSET
        if not isinstance(self.power_type, Unset):
            power_type = self.power_type.value

        max_voltage = self.max_voltage

        max_amperage = self.max_amperage

        max_electric_power = self.max_electric_power

        calc_max_electric_power = self.calc_max_electric_power

        last_updated: Union[Unset, str] = UNSET
        if not isinstance(self.last_updated, Unset):
            last_updated = self.last_updated.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if standard is not UNSET:
            field_dict["standard"] = standard
        if format_ is not UNSET:
            field_dict["format"] = format_
        if power_type is not UNSET:
            field_dict["power_type"] = power_type
        if max_voltage is not UNSET:
            field_dict["max_voltage"] = max_voltage
        if max_amperage is not UNSET:
            field_dict["max_amperage"] = max_amperage
        if max_electric_power is not UNSET:
            field_dict["max_electric_power"] = max_electric_power
        if calc_max_electric_power is not UNSET:
            field_dict["calc_max_electric_power"] = calc_max_electric_power
        if last_updated is not UNSET:
            field_dict["last_updated"] = last_updated

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", UNSET)

        _standard = d.pop("standard", UNSET)
        standard: Union[Unset, ConnectorDtoStandard]
        if isinstance(_standard, Unset) or _standard is None:
            standard = UNSET
        else:
            standard = ConnectorDtoStandard(_standard)

        _format_ = d.pop("format", UNSET)
        format_: Union[Unset, ConnectorDtoFormat]
        if isinstance(_format_, Unset) or _format_ is None:
            format_ = UNSET
        else:
            format_ = ConnectorDtoFormat(_format_)

        _power_type = d.pop("power_type", UNSET)
        power_type: Union[Unset, ConnectorDtoPowerType]
        if isinstance(_power_type, Unset) or _power_type is None:
            power_type = UNSET
        else:
            power_type = ConnectorDtoPowerType(_power_type)

        max_voltage = d.pop("max_voltage", UNSET)

        max_amperage = d.pop("max_amperage", UNSET)

        max_electric_power = d.pop("max_electric_power", UNSET)

        calc_max_electric_power = d.pop("calc_max_electric_power", UNSET)

        _last_updated = d.pop("last_updated", UNSET)
        last_updated: Union[Unset, datetime.datetime]
        if isinstance(_last_updated, Unset) or _last_updated is None:
            last_updated = UNSET
        else:
            last_updated = isoparse(_last_updated)

        connector_dto = cls(
            id=id,
            standard=standard,
            format_=format_,
            power_type=power_type,
            max_voltage=max_voltage,
            max_amperage=max_amperage,
            max_electric_power=max_electric_power,
            calc_max_electric_power=calc_max_electric_power,
            last_updated=last_updated,
        )

        connector_dto.additional_properties = d
        return connector_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from dateutil.parser import isoparse
import datetime
from typing import Union

if TYPE_CHECKING:
    from ..models.reimbursement_cdr_location_dto import ReimbursementCdrLocationDto
    from ..models.reimbursement_customer_share_dto import ReimbursementCustomerShareDto
    from ..models.reimburse_started_by_info_dto import ReimburseStartedByInfoDto
    from ..models.price_info_dto import PriceInfoDto


T = TypeVar("T", bound="ReimbursementCdrDto")


@_attrs_define
class ReimbursementCdrDto:
    """
    Attributes:
        id (Union[Unset, str]):
        tenant_id (Union[Unset, str]):
        charge_point_id (Union[Unset, str]):
        connector_id (Union[Unset, int]):
        location_id (Union[Unset, str]):
        evse_id (Union[Unset, str]):
        location (Union[Unset, ReimbursementCdrLocationDto]):
        start_datetime (Union[Unset, datetime.datetime]):
        end_date_time (Union[Unset, datetime.datetime]):
        session_id (Union[Unset, str]):
        started_by_info (Union[Unset, ReimburseStartedByInfoDto]):
        meter_start_in_wh (Union[Unset, int]):
        meter_stop_in_wh (Union[Unset, int]):
        total_energy_in_kwh (Union[Unset, float]):
        total_time_in_hours (Union[Unset, float]):
        total_price (Union[Unset, float]):
        created (Union[Unset, datetime.datetime]):
        last_updated (Union[Unset, datetime.datetime]):
        ou (Union[Unset, str]):
        ou_id (Union[Unset, str]):
        ou_name (Union[Unset, str]):
        reimburse_tariff_id (Union[Unset, str]):
        reimburse_tariff_name (Union[Unset, str]):
        reimburse_tariff_price (Union[Unset, float]):
        reimburse_tariff_calculated (Union[Unset, bool]):
        reimburse_price_calculated_on (Union[Unset, datetime.datetime]):
        bank_account (Union[Unset, str]):
        bank_account_created_on (Union[Unset, datetime.datetime]):
        bank_account_valid_from (Union[Unset, datetime.datetime]):
        reimburse_tariff_original_price (Union[Unset, float]):
        has_guest_charging_reimbursement_fee (Union[Unset, bool]):
        reimburse_tenant_fee (Union[Unset, float]):
        tenant_fee_calculated (Union[Unset, bool]):
        tariff_distribution_id (Union[Unset, str]):
        price_info (Union[Unset, PriceInfoDto]):
        customer_share (Union[Unset, float]):
        energy_compensation (Union[Unset, float]):
        reimbursement_customer_share (Union[Unset, ReimbursementCustomerShareDto]):
        local_start_date_time (Union[Unset, datetime.datetime]):
        local_end_date_time (Union[Unset, datetime.datetime]):
    """

    id: Union[Unset, str] = UNSET
    tenant_id: Union[Unset, str] = UNSET
    charge_point_id: Union[Unset, str] = UNSET
    connector_id: Union[Unset, int] = UNSET
    location_id: Union[Unset, str] = UNSET
    evse_id: Union[Unset, str] = UNSET
    location: Union[Unset, "ReimbursementCdrLocationDto"] = UNSET
    start_datetime: Union[Unset, datetime.datetime] = UNSET
    end_date_time: Union[Unset, datetime.datetime] = UNSET
    session_id: Union[Unset, str] = UNSET
    started_by_info: Union[Unset, "ReimburseStartedByInfoDto"] = UNSET
    meter_start_in_wh: Union[Unset, int] = UNSET
    meter_stop_in_wh: Union[Unset, int] = UNSET
    total_energy_in_kwh: Union[Unset, float] = UNSET
    total_time_in_hours: Union[Unset, float] = UNSET
    total_price: Union[Unset, float] = UNSET
    created: Union[Unset, datetime.datetime] = UNSET
    last_updated: Union[Unset, datetime.datetime] = UNSET
    ou: Union[Unset, str] = UNSET
    ou_id: Union[Unset, str] = UNSET
    ou_name: Union[Unset, str] = UNSET
    reimburse_tariff_id: Union[Unset, str] = UNSET
    reimburse_tariff_name: Union[Unset, str] = UNSET
    reimburse_tariff_price: Union[Unset, float] = UNSET
    reimburse_tariff_calculated: Union[Unset, bool] = UNSET
    reimburse_price_calculated_on: Union[Unset, datetime.datetime] = UNSET
    bank_account: Union[Unset, str] = UNSET
    bank_account_created_on: Union[Unset, datetime.datetime] = UNSET
    bank_account_valid_from: Union[Unset, datetime.datetime] = UNSET
    reimburse_tariff_original_price: Union[Unset, float] = UNSET
    has_guest_charging_reimbursement_fee: Union[Unset, bool] = UNSET
    reimburse_tenant_fee: Union[Unset, float] = UNSET
    tenant_fee_calculated: Union[Unset, bool] = UNSET
    tariff_distribution_id: Union[Unset, str] = UNSET
    price_info: Union[Unset, "PriceInfoDto"] = UNSET
    customer_share: Union[Unset, float] = UNSET
    energy_compensation: Union[Unset, float] = UNSET
    reimbursement_customer_share: Union[Unset, "ReimbursementCustomerShareDto"] = UNSET
    local_start_date_time: Union[Unset, datetime.datetime] = UNSET
    local_end_date_time: Union[Unset, datetime.datetime] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        tenant_id = self.tenant_id

        charge_point_id = self.charge_point_id

        connector_id = self.connector_id

        location_id = self.location_id

        evse_id = self.evse_id

        location: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.location, Unset):
            location = self.location.to_dict()

        start_datetime: Union[Unset, str] = UNSET
        if not isinstance(self.start_datetime, Unset):
            start_datetime = self.start_datetime.isoformat()

        end_date_time: Union[Unset, str] = UNSET
        if not isinstance(self.end_date_time, Unset):
            end_date_time = self.end_date_time.isoformat()

        session_id = self.session_id

        started_by_info: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.started_by_info, Unset):
            started_by_info = self.started_by_info.to_dict()

        meter_start_in_wh = self.meter_start_in_wh

        meter_stop_in_wh = self.meter_stop_in_wh

        total_energy_in_kwh = self.total_energy_in_kwh

        total_time_in_hours = self.total_time_in_hours

        total_price = self.total_price

        created: Union[Unset, str] = UNSET
        if not isinstance(self.created, Unset):
            created = self.created.isoformat()

        last_updated: Union[Unset, str] = UNSET
        if not isinstance(self.last_updated, Unset):
            last_updated = self.last_updated.isoformat()

        ou = self.ou

        ou_id = self.ou_id

        ou_name = self.ou_name

        reimburse_tariff_id = self.reimburse_tariff_id

        reimburse_tariff_name = self.reimburse_tariff_name

        reimburse_tariff_price = self.reimburse_tariff_price

        reimburse_tariff_calculated = self.reimburse_tariff_calculated

        reimburse_price_calculated_on: Union[Unset, str] = UNSET
        if not isinstance(self.reimburse_price_calculated_on, Unset):
            reimburse_price_calculated_on = (
                self.reimburse_price_calculated_on.isoformat()
            )

        bank_account = self.bank_account

        bank_account_created_on: Union[Unset, str] = UNSET
        if not isinstance(self.bank_account_created_on, Unset):
            bank_account_created_on = self.bank_account_created_on.isoformat()

        bank_account_valid_from: Union[Unset, str] = UNSET
        if not isinstance(self.bank_account_valid_from, Unset):
            bank_account_valid_from = self.bank_account_valid_from.isoformat()

        reimburse_tariff_original_price = self.reimburse_tariff_original_price

        has_guest_charging_reimbursement_fee = self.has_guest_charging_reimbursement_fee

        reimburse_tenant_fee = self.reimburse_tenant_fee

        tenant_fee_calculated = self.tenant_fee_calculated

        tariff_distribution_id = self.tariff_distribution_id

        price_info: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.price_info, Unset):
            price_info = self.price_info.to_dict()

        customer_share = self.customer_share

        energy_compensation = self.energy_compensation

        reimbursement_customer_share: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.reimbursement_customer_share, Unset):
            reimbursement_customer_share = self.reimbursement_customer_share.to_dict()

        local_start_date_time: Union[Unset, str] = UNSET
        if not isinstance(self.local_start_date_time, Unset):
            local_start_date_time = self.local_start_date_time.isoformat()

        local_end_date_time: Union[Unset, str] = UNSET
        if not isinstance(self.local_end_date_time, Unset):
            local_end_date_time = self.local_end_date_time.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if tenant_id is not UNSET:
            field_dict["tenantId"] = tenant_id
        if charge_point_id is not UNSET:
            field_dict["chargePointId"] = charge_point_id
        if connector_id is not UNSET:
            field_dict["connectorId"] = connector_id
        if location_id is not UNSET:
            field_dict["locationId"] = location_id
        if evse_id is not UNSET:
            field_dict["evseId"] = evse_id
        if location is not UNSET:
            field_dict["location"] = location
        if start_datetime is not UNSET:
            field_dict["startDatetime"] = start_datetime
        if end_date_time is not UNSET:
            field_dict["endDateTime"] = end_date_time
        if session_id is not UNSET:
            field_dict["sessionId"] = session_id
        if started_by_info is not UNSET:
            field_dict["startedByInfo"] = started_by_info
        if meter_start_in_wh is not UNSET:
            field_dict["meterStartInWh"] = meter_start_in_wh
        if meter_stop_in_wh is not UNSET:
            field_dict["meterStopInWh"] = meter_stop_in_wh
        if total_energy_in_kwh is not UNSET:
            field_dict["totalEnergyInKwh"] = total_energy_in_kwh
        if total_time_in_hours is not UNSET:
            field_dict["totalTimeInHours"] = total_time_in_hours
        if total_price is not UNSET:
            field_dict["totalPrice"] = total_price
        if created is not UNSET:
            field_dict["created"] = created
        if last_updated is not UNSET:
            field_dict["lastUpdated"] = last_updated
        if ou is not UNSET:
            field_dict["ou"] = ou
        if ou_id is not UNSET:
            field_dict["ouId"] = ou_id
        if ou_name is not UNSET:
            field_dict["ouName"] = ou_name
        if reimburse_tariff_id is not UNSET:
            field_dict["reimburseTariffId"] = reimburse_tariff_id
        if reimburse_tariff_name is not UNSET:
            field_dict["reimburseTariffName"] = reimburse_tariff_name
        if reimburse_tariff_price is not UNSET:
            field_dict["reimburseTariffPrice"] = reimburse_tariff_price
        if reimburse_tariff_calculated is not UNSET:
            field_dict["reimburseTariffCalculated"] = reimburse_tariff_calculated
        if reimburse_price_calculated_on is not UNSET:
            field_dict["reimbursePriceCalculatedOn"] = reimburse_price_calculated_on
        if bank_account is not UNSET:
            field_dict["bankAccount"] = bank_account
        if bank_account_created_on is not UNSET:
            field_dict["bankAccountCreatedOn"] = bank_account_created_on
        if bank_account_valid_from is not UNSET:
            field_dict["bankAccountValidFrom"] = bank_account_valid_from
        if reimburse_tariff_original_price is not UNSET:
            field_dict["reimburseTariffOriginalPrice"] = reimburse_tariff_original_price
        if has_guest_charging_reimbursement_fee is not UNSET:
            field_dict["hasGuestChargingReimbursementFee"] = (
                has_guest_charging_reimbursement_fee
            )
        if reimburse_tenant_fee is not UNSET:
            field_dict["reimburseTenantFee"] = reimburse_tenant_fee
        if tenant_fee_calculated is not UNSET:
            field_dict["tenantFeeCalculated"] = tenant_fee_calculated
        if tariff_distribution_id is not UNSET:
            field_dict["tariffDistributionId"] = tariff_distribution_id
        if price_info is not UNSET:
            field_dict["priceInfo"] = price_info
        if customer_share is not UNSET:
            field_dict["customerShare"] = customer_share
        if energy_compensation is not UNSET:
            field_dict["energyCompensation"] = energy_compensation
        if reimbursement_customer_share is not UNSET:
            field_dict["reimbursementCustomerShare"] = reimbursement_customer_share
        if local_start_date_time is not UNSET:
            field_dict["localStartDateTime"] = local_start_date_time
        if local_end_date_time is not UNSET:
            field_dict["localEndDateTime"] = local_end_date_time

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.reimbursement_cdr_location_dto import ReimbursementCdrLocationDto
        from ..models.reimbursement_customer_share_dto import (
            ReimbursementCustomerShareDto,
        )
        from ..models.reimburse_started_by_info_dto import ReimburseStartedByInfoDto
        from ..models.price_info_dto import PriceInfoDto

        d = src_dict.copy()
        id = d.pop("id", UNSET)

        tenant_id = d.pop("tenantId", UNSET)

        charge_point_id = d.pop("chargePointId", UNSET)

        connector_id = d.pop("connectorId", UNSET)

        location_id = d.pop("locationId", UNSET)

        evse_id = d.pop("evseId", UNSET)

        _location = d.pop("location", UNSET)
        location: Union[Unset, ReimbursementCdrLocationDto]
        if isinstance(_location, Unset) or _location is None:
            location = UNSET
        else:
            location = ReimbursementCdrLocationDto.from_dict(_location)

        _start_datetime = d.pop("startDatetime", UNSET)
        start_datetime: Union[Unset, datetime.datetime]
        if isinstance(_start_datetime, Unset) or _start_datetime is None:
            start_datetime = UNSET
        else:
            start_datetime = isoparse(_start_datetime)

        _end_date_time = d.pop("endDateTime", UNSET)
        end_date_time: Union[Unset, datetime.datetime]
        if isinstance(_end_date_time, Unset) or _end_date_time is None:
            end_date_time = UNSET
        else:
            end_date_time = isoparse(_end_date_time)

        session_id = d.pop("sessionId", UNSET)

        _started_by_info = d.pop("startedByInfo", UNSET)
        started_by_info: Union[Unset, ReimburseStartedByInfoDto]
        if isinstance(_started_by_info, Unset) or _started_by_info is None:
            started_by_info = UNSET
        else:
            started_by_info = ReimburseStartedByInfoDto.from_dict(_started_by_info)

        meter_start_in_wh = d.pop("meterStartInWh", UNSET)

        meter_stop_in_wh = d.pop("meterStopInWh", UNSET)

        total_energy_in_kwh = d.pop("totalEnergyInKwh", UNSET)

        total_time_in_hours = d.pop("totalTimeInHours", UNSET)

        total_price = d.pop("totalPrice", UNSET)

        _created = d.pop("created", UNSET)
        created: Union[Unset, datetime.datetime]
        if isinstance(_created, Unset) or _created is None:
            created = UNSET
        else:
            created = isoparse(_created)

        _last_updated = d.pop("lastUpdated", UNSET)
        last_updated: Union[Unset, datetime.datetime]
        if isinstance(_last_updated, Unset) or _last_updated is None:
            last_updated = UNSET
        else:
            last_updated = isoparse(_last_updated)

        ou = d.pop("ou", UNSET)

        ou_id = d.pop("ouId", UNSET)

        ou_name = d.pop("ouName", UNSET)

        reimburse_tariff_id = d.pop("reimburseTariffId", UNSET)

        reimburse_tariff_name = d.pop("reimburseTariffName", UNSET)

        reimburse_tariff_price = d.pop("reimburseTariffPrice", UNSET)

        reimburse_tariff_calculated = d.pop("reimburseTariffCalculated", UNSET)

        _reimburse_price_calculated_on = d.pop("reimbursePriceCalculatedOn", UNSET)
        reimburse_price_calculated_on: Union[Unset, datetime.datetime]
        if isinstance(_reimburse_price_calculated_on, Unset) or _reimburse_price_calculated_on is None:
            reimburse_price_calculated_on = UNSET
        else:
            reimburse_price_calculated_on = isoparse(_reimburse_price_calculated_on)

        bank_account = d.pop("bankAccount", UNSET)

        _bank_account_created_on = d.pop("bankAccountCreatedOn", UNSET)
        bank_account_created_on: Union[Unset, datetime.datetime]
        if isinstance(_bank_account_created_on, Unset) or _bank_account_created_on is None:
            bank_account_created_on = UNSET
        else:
            bank_account_created_on = isoparse(_bank_account_created_on)

        _bank_account_valid_from = d.pop("bankAccountValidFrom", UNSET)
        bank_account_valid_from: Union[Unset, datetime.datetime]
        if isinstance(_bank_account_valid_from, Unset) or _bank_account_valid_from is None:
            bank_account_valid_from = UNSET
        else:
            bank_account_valid_from = isoparse(_bank_account_valid_from)

        reimburse_tariff_original_price = d.pop("reimburseTariffOriginalPrice", UNSET)

        has_guest_charging_reimbursement_fee = d.pop(
            "hasGuestChargingReimbursementFee", UNSET
        )

        reimburse_tenant_fee = d.pop("reimburseTenantFee", UNSET)

        tenant_fee_calculated = d.pop("tenantFeeCalculated", UNSET)

        tariff_distribution_id = d.pop("tariffDistributionId", UNSET)

        _price_info = d.pop("priceInfo", UNSET)
        price_info: Union[Unset, PriceInfoDto]
        if isinstance(_price_info, Unset) or _price_info is None:
            price_info = UNSET
        else:
            price_info = PriceInfoDto.from_dict(_price_info)

        customer_share = d.pop("customerShare", UNSET)

        energy_compensation = d.pop("energyCompensation", UNSET)

        _reimbursement_customer_share = d.pop("reimbursementCustomerShare", UNSET)
        reimbursement_customer_share: Union[Unset, ReimbursementCustomerShareDto]
        if isinstance(_reimbursement_customer_share, Unset) or _reimbursement_customer_share is None:
            reimbursement_customer_share = UNSET
        else:
            reimbursement_customer_share = ReimbursementCustomerShareDto.from_dict(
                _reimbursement_customer_share
            )

        _local_start_date_time = d.pop("localStartDateTime", UNSET)
        local_start_date_time: Union[Unset, datetime.datetime]
        if isinstance(_local_start_date_time, Unset) or _local_start_date_time is None:
            local_start_date_time = UNSET
        else:
            local_start_date_time = isoparse(_local_start_date_time)

        _local_end_date_time = d.pop("localEndDateTime", UNSET)
        local_end_date_time: Union[Unset, datetime.datetime]
        if isinstance(_local_end_date_time, Unset) or _local_end_date_time is None:
            local_end_date_time = UNSET
        else:
            local_end_date_time = isoparse(_local_end_date_time)

        reimbursement_cdr_dto = cls(
            id=id,
            tenant_id=tenant_id,
            charge_point_id=charge_point_id,
            connector_id=connector_id,
            location_id=location_id,
            evse_id=evse_id,
            location=location,
            start_datetime=start_datetime,
            end_date_time=end_date_time,
            session_id=session_id,
            started_by_info=started_by_info,
            meter_start_in_wh=meter_start_in_wh,
            meter_stop_in_wh=meter_stop_in_wh,
            total_energy_in_kwh=total_energy_in_kwh,
            total_time_in_hours=total_time_in_hours,
            total_price=total_price,
            created=created,
            last_updated=last_updated,
            ou=ou,
            ou_id=ou_id,
            ou_name=ou_name,
            reimburse_tariff_id=reimburse_tariff_id,
            reimburse_tariff_name=reimburse_tariff_name,
            reimburse_tariff_price=reimburse_tariff_price,
            reimburse_tariff_calculated=reimburse_tariff_calculated,
            reimburse_price_calculated_on=reimburse_price_calculated_on,
            bank_account=bank_account,
            bank_account_created_on=bank_account_created_on,
            bank_account_valid_from=bank_account_valid_from,
            reimburse_tariff_original_price=reimburse_tariff_original_price,
            has_guest_charging_reimbursement_fee=has_guest_charging_reimbursement_fee,
            reimburse_tenant_fee=reimburse_tenant_fee,
            tenant_fee_calculated=tenant_fee_calculated,
            tariff_distribution_id=tariff_distribution_id,
            price_info=price_info,
            customer_share=customer_share,
            energy_compensation=energy_compensation,
            reimbursement_customer_share=reimbursement_customer_share,
            local_start_date_time=local_start_date_time,
            local_end_date_time=local_end_date_time,
        )

        reimbursement_cdr_dto.additional_properties = d
        return reimbursement_cdr_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union


T = TypeVar("T", bound="DataTransferRequest")


@_attrs_define
class DataTransferRequest:
    """
    Attributes:
        vendor_id (str):
        message_id (Union[Unset, str]):
        data (Union[Unset, str]):
    """

    vendor_id: str
    message_id: Union[Unset, str] = UNSET
    data: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        vendor_id = self.vendor_id

        message_id = self.message_id

        data = self.data

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "vendorId": vendor_id,
            }
        )
        if message_id is not UNSET:
            field_dict["messageId"] = message_id
        if data is not UNSET:
            field_dict["data"] = data

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        vendor_id = d.pop("vendorId")

        message_id = d.pop("messageId", UNSET)

        data = d.pop("data", UNSET)

        data_transfer_request = cls(
            vendor_id=vendor_id,
            message_id=message_id,
            data=data,
        )

        data_transfer_request.additional_properties = d
        return data_transfer_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class GetAllTariffsOrderBy(str, Enum):
    KWH_PRICE = "kwH_PRICE"
    NAME = "name"
    START_PRICE = "starT_PRICE"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from dateutil.parser import isoparse
import datetime
from typing import Union
from ..types import File, FileJsonType
from io import BytesIO

if TYPE_CHECKING:
    from ..models.entity_tag_header_value import EntityTagHeaderValue


T = TypeVar("T", bound="FileContentResult")


@_attrs_define
class FileContentResult:
    """
    Attributes:
        file_contents (Union[Unset, File]):
        content_type (Union[Unset, str]):
        file_download_name (Union[Unset, str]):
        last_modified (Union[Unset, datetime.datetime]):
        entity_tag (Union[Unset, EntityTagHeaderValue]):
        enable_range_processing (Union[Unset, bool]):
    """

    file_contents: Union[Unset, File] = UNSET
    content_type: Union[Unset, str] = UNSET
    file_download_name: Union[Unset, str] = UNSET
    last_modified: Union[Unset, datetime.datetime] = UNSET
    entity_tag: Union[Unset, "EntityTagHeaderValue"] = UNSET
    enable_range_processing: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        file_contents: Union[Unset, FileJsonType] = UNSET
        if not isinstance(self.file_contents, Unset):
            file_contents = self.file_contents.to_tuple()

        content_type = self.content_type

        file_download_name = self.file_download_name

        last_modified: Union[Unset, str] = UNSET
        if not isinstance(self.last_modified, Unset):
            last_modified = self.last_modified.isoformat()

        entity_tag: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.entity_tag, Unset):
            entity_tag = self.entity_tag.to_dict()

        enable_range_processing = self.enable_range_processing

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if file_contents is not UNSET:
            field_dict["fileContents"] = file_contents
        if content_type is not UNSET:
            field_dict["contentType"] = content_type
        if file_download_name is not UNSET:
            field_dict["fileDownloadName"] = file_download_name
        if last_modified is not UNSET:
            field_dict["lastModified"] = last_modified
        if entity_tag is not UNSET:
            field_dict["entityTag"] = entity_tag
        if enable_range_processing is not UNSET:
            field_dict["enableRangeProcessing"] = enable_range_processing

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.entity_tag_header_value import EntityTagHeaderValue

        d = src_dict.copy()
        _file_contents = d.pop("fileContents", UNSET)
        file_contents: Union[Unset, File]
        if isinstance(_file_contents, Unset) or _file_contents is None:
            file_contents = UNSET
        else:
            file_contents = File(payload=BytesIO(_file_contents))

        content_type = d.pop("contentType", UNSET)

        file_download_name = d.pop("fileDownloadName", UNSET)

        _last_modified = d.pop("lastModified", UNSET)
        last_modified: Union[Unset, datetime.datetime]
        if isinstance(_last_modified, Unset) or _last_modified is None:
            last_modified = UNSET
        else:
            last_modified = isoparse(_last_modified)

        _entity_tag = d.pop("entityTag", UNSET)
        entity_tag: Union[Unset, EntityTagHeaderValue]
        if isinstance(_entity_tag, Unset) or _entity_tag is None:
            entity_tag = UNSET
        else:
            entity_tag = EntityTagHeaderValue.from_dict(_entity_tag)

        enable_range_processing = d.pop("enableRangeProcessing", UNSET)

        file_content_result = cls(
            file_contents=file_contents,
            content_type=content_type,
            file_download_name=file_download_name,
            last_modified=last_modified,
            entity_tag=entity_tag,
            enable_range_processing=enable_range_processing,
        )

        file_content_result.additional_properties = d
        return file_content_result

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from dateutil.parser import isoparse
import datetime
from typing import Union


T = TypeVar("T", bound="ChargepointConfigurationItemsDto")


@_attrs_define
class ChargepointConfigurationItemsDto:
    """
    Attributes:
        id (Union[Unset, str]):
        read_only (Union[Unset, bool]):
        value (Union[Unset, str]):
        created (Union[Unset, datetime.datetime]):
        modified (Union[Unset, datetime.datetime]):
        deleted (Union[Unset, datetime.datetime]):
    """

    id: Union[Unset, str] = UNSET
    read_only: Union[Unset, bool] = UNSET
    value: Union[Unset, str] = UNSET
    created: Union[Unset, datetime.datetime] = UNSET
    modified: Union[Unset, datetime.datetime] = UNSET
    deleted: Union[Unset, datetime.datetime] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        read_only = self.read_only

        value = self.value

        created: Union[Unset, str] = UNSET
        if not isinstance(self.created, Unset):
            created = self.created.isoformat()

        modified: Union[Unset, str] = UNSET
        if not isinstance(self.modified, Unset):
            modified = self.modified.isoformat()

        deleted: Union[Unset, str] = UNSET
        if not isinstance(self.deleted, Unset):
            deleted = self.deleted.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if read_only is not UNSET:
            field_dict["readOnly"] = read_only
        if value is not UNSET:
            field_dict["value"] = value
        if created is not UNSET:
            field_dict["created"] = created
        if modified is not UNSET:
            field_dict["modified"] = modified
        if deleted is not UNSET:
            field_dict["deleted"] = deleted

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", UNSET)

        read_only = d.pop("readOnly", UNSET)

        value = d.pop("value", UNSET)

        _created = d.pop("created", UNSET)
        created: Union[Unset, datetime.datetime]
        if isinstance(_created, Unset) or _created is None:
            created = UNSET
        else:
            created = isoparse(_created)

        _modified = d.pop("modified", UNSET)
        modified: Union[Unset, datetime.datetime]
        if isinstance(_modified, Unset) or _modified is None:
            modified = UNSET
        else:
            modified = isoparse(_modified)

        _deleted = d.pop("deleted", UNSET)
        deleted: Union[Unset, datetime.datetime]
        if isinstance(_deleted, Unset) or _deleted is None:
            deleted = UNSET
        else:
            deleted = isoparse(_deleted)

        chargepoint_configuration_items_dto = cls(
            id=id,
            read_only=read_only,
            value=value,
            created=created,
            modified=modified,
            deleted=deleted,
        )

        chargepoint_configuration_items_dto.additional_properties = d
        return chargepoint_configuration_items_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.cs_charging_profiles_charging_profile_kind import (
    CsChargingProfilesChargingProfileKind,
)
from dateutil.parser import isoparse
from ..models.cs_charging_profiles_recurrency_kind import (
    CsChargingProfilesRecurrencyKind,
)
import datetime
from ..models.cs_charging_profiles_charging_profile_purpose import (
    CsChargingProfilesChargingProfilePurpose,
)
from typing import Union

if TYPE_CHECKING:
    from ..models.charging_schedule import ChargingSchedule


T = TypeVar("T", bound="CsChargingProfiles")


@_attrs_define
class CsChargingProfiles:
    """
    Attributes:
        charging_profile_id (int):
        stack_level (int):
        charging_profile_purpose (CsChargingProfilesChargingProfilePurpose):  Default:
            CsChargingProfilesChargingProfilePurpose.CHARGEPOINTMAXPROFILE.
        charging_profile_kind (CsChargingProfilesChargingProfileKind):  Default:
            CsChargingProfilesChargingProfileKind.ABSOLUTE.
        charging_schedule (ChargingSchedule):
        transaction_id (Union[Unset, int]):
        recurrency_kind (Union[Unset, CsChargingProfilesRecurrencyKind]):  Default:
            CsChargingProfilesRecurrencyKind.DAILY.
        valid_from (Union[Unset, datetime.datetime]):
        valid_to (Union[Unset, datetime.datetime]):
    """

    charging_profile_id: int
    stack_level: int
    charging_schedule: "ChargingSchedule"
    charging_profile_purpose: CsChargingProfilesChargingProfilePurpose = (
        CsChargingProfilesChargingProfilePurpose.CHARGEPOINTMAXPROFILE
    )
    charging_profile_kind: CsChargingProfilesChargingProfileKind = (
        CsChargingProfilesChargingProfileKind.ABSOLUTE
    )
    transaction_id: Union[Unset, int] = UNSET
    recurrency_kind: Union[Unset, CsChargingProfilesRecurrencyKind] = (
        CsChargingProfilesRecurrencyKind.DAILY
    )
    valid_from: Union[Unset, datetime.datetime] = UNSET
    valid_to: Union[Unset, datetime.datetime] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        charging_profile_id = self.charging_profile_id

        stack_level = self.stack_level

        charging_profile_purpose = self.charging_profile_purpose.value

        charging_profile_kind = self.charging_profile_kind.value

        charging_schedule = self.charging_schedule.to_dict()

        transaction_id = self.transaction_id

        recurrency_kind: Union[Unset, str] = UNSET
        if not isinstance(self.recurrency_kind, Unset):
            recurrency_kind = self.recurrency_kind.value

        valid_from: Union[Unset, str] = UNSET
        if not isinstance(self.valid_from, Unset):
            valid_from = self.valid_from.isoformat()

        valid_to: Union[Unset, str] = UNSET
        if not isinstance(self.valid_to, Unset):
            valid_to = self.valid_to.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "chargingProfileId": charging_profile_id,
                "stackLevel": stack_level,
                "chargingProfilePurpose": charging_profile_purpose,
                "chargingProfileKind": charging_profile_kind,
                "chargingSchedule": charging_schedule,
            }
        )
        if transaction_id is not UNSET:
            field_dict["transactionId"] = transaction_id
        if recurrency_kind is not UNSET:
            field_dict["recurrencyKind"] = recurrency_kind
        if valid_from is not UNSET:
            field_dict["validFrom"] = valid_from
        if valid_to is not UNSET:
            field_dict["validTo"] = valid_to

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.charging_schedule import ChargingSchedule

        d = src_dict.copy()
        charging_profile_id = d.pop("chargingProfileId")

        stack_level = d.pop("stackLevel")

        charging_profile_purpose = CsChargingProfilesChargingProfilePurpose(
            d.pop("chargingProfilePurpose")
        )

        charging_profile_kind = CsChargingProfilesChargingProfileKind(
            d.pop("chargingProfileKind")
        )

        charging_schedule = ChargingSchedule.from_dict(d.pop("chargingSchedule"))

        transaction_id = d.pop("transactionId", UNSET)

        _recurrency_kind = d.pop("recurrencyKind", UNSET)
        recurrency_kind: Union[Unset, CsChargingProfilesRecurrencyKind]
        if isinstance(_recurrency_kind, Unset) or _recurrency_kind is None:
            recurrency_kind = UNSET
        else:
            recurrency_kind = CsChargingProfilesRecurrencyKind(_recurrency_kind)

        _valid_from = d.pop("validFrom", UNSET)
        valid_from: Union[Unset, datetime.datetime]
        if isinstance(_valid_from, Unset) or _valid_from is None:
            valid_from = UNSET
        else:
            valid_from = isoparse(_valid_from)

        _valid_to = d.pop("validTo", UNSET)
        valid_to: Union[Unset, datetime.datetime]
        if isinstance(_valid_to, Unset) or _valid_to is None:
            valid_to = UNSET
        else:
            valid_to = isoparse(_valid_to)

        cs_charging_profiles = cls(
            charging_profile_id=charging_profile_id,
            stack_level=stack_level,
            charging_profile_purpose=charging_profile_purpose,
            charging_profile_kind=charging_profile_kind,
            charging_schedule=charging_schedule,
            transaction_id=transaction_id,
            recurrency_kind=recurrency_kind,
            valid_from=valid_from,
            valid_to=valid_to,
        )

        cs_charging_profiles.additional_properties = d
        return cs_charging_profiles

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union
from ..models.token_info_dto_token_type import TokenInfoDtoTokenType


T = TypeVar("T", bound="TokenInfoDto")


@_attrs_define
class TokenInfoDto:
    """
    Attributes:
        id_tag (Union[Unset, str]):
        contract_id (Union[Unset, str]):
        token_type (Union[Unset, TokenInfoDtoTokenType]):  Default: TokenInfoDtoTokenType.ADHOCUSER.
    """

    id_tag: Union[Unset, str] = UNSET
    contract_id: Union[Unset, str] = UNSET
    token_type: Union[Unset, TokenInfoDtoTokenType] = TokenInfoDtoTokenType.ADHOCUSER
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id_tag = self.id_tag

        contract_id = self.contract_id

        token_type: Union[Unset, str] = UNSET
        if not isinstance(self.token_type, Unset):
            token_type = self.token_type.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id_tag is not UNSET:
            field_dict["idTag"] = id_tag
        if contract_id is not UNSET:
            field_dict["contractId"] = contract_id
        if token_type is not UNSET:
            field_dict["tokenType"] = token_type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id_tag = d.pop("idTag", UNSET)

        contract_id = d.pop("contractId", UNSET)

        _token_type = d.pop("tokenType", UNSET)
        token_type: Union[Unset, TokenInfoDtoTokenType]
        if isinstance(_token_type, Unset) or _token_type is None:
            token_type = UNSET
        else:
            token_type = TokenInfoDtoTokenType(_token_type)

        token_info_dto = cls(
            id_tag=id_tag,
            contract_id=contract_id,
            token_type=token_type,
        )

        token_info_dto.additional_properties = d
        return token_info_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
